/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_IMX374_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_IMX374_H_

#include "cam_sensor_dev.h"

int num_front_mipi_setting = 2;

/*
Mode A : Full mode (860.8 / 995.2 / 1011.2 / 1046.4)
Mode B : 4k2k 60fps mode (1196.8 / 1011.2 / 1139.2 / 1094.4)
*/

/*************************************/
/* ========== Mode A =============== */
/*************************************/

enum {

	CAM_IMX374_SET_A_FULL_860_MHZ = 0,
	CAM_IMX374_SET_A_FULL_995_MHZ = 1,
	CAM_IMX374_SET_A_FULL_1011_MHZ = 2,
	CAM_IMX374_SET_A_FULL_1046_MHZ = 3,
};

struct cam_sensor_i2c_reg_array MIPI_FULL_1046_MHZ_REG_ARRAY[] = {
    {0x030F, 0x47, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_FULL_860_MHZ_REG_ARRAY[] = {
    {0x030F, 0x0D, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_FULL_995_MHZ_REG_ARRAY[] = {
    {0x030F, 0x37, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_FULL_1011_MHZ_REG_ARRAY[] = {
    {0x030F, 0x3C, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_A_mipi_FULL_1046_mhz[] = {
    { MIPI_FULL_1046_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_1046_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_A_mipi_FULL_860_mhz[] = {
    { MIPI_FULL_860_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_860_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_A_mipi_FULL_995_mhz[] = {
    { MIPI_FULL_995_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_995_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_A_mipi_FULL_1011_mhz[] = {
    { MIPI_FULL_1011_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_1011_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx374_setfile_A_mipi_setting_FULL[] = {
	{ "860 Mhz",
	  sensor_imx374_setfile_A_mipi_FULL_860_mhz, ARRAY_SIZE(sensor_imx374_setfile_A_mipi_FULL_860_mhz) },
	{ "995 Mhz",
	  sensor_imx374_setfile_A_mipi_FULL_995_mhz, ARRAY_SIZE(sensor_imx374_setfile_A_mipi_FULL_995_mhz) },
	{ "1011 Mhz",
	  sensor_imx374_setfile_A_mipi_FULL_1011_mhz, ARRAY_SIZE(sensor_imx374_setfile_A_mipi_FULL_1011_mhz) },
	{ "1046 Mhz",
	  sensor_imx374_setfile_A_mipi_FULL_1046_mhz, ARRAY_SIZE(sensor_imx374_setfile_A_mipi_FULL_1046_mhz) },
};

static const struct cam_mipi_channel sensor_imx374_setfile_A_mipi_channel_FULL[] = {
 	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10582, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10583, 10722, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10723, 10735, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10736, 10797, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10798, 10838, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9665, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9666, 9721, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9722, 9861, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9862, 9914, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9915, 9936, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9937, 9938, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1200, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1201, 1341, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1342, 1368, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1369, 1416, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1417, 1513, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1557, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1558, 1697, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1698, 1710, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1711, 1738, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4458, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2273, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2274, 2289, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2290, 2429, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2430, 2504, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2505, 2563, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2996, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2997, 3064, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3065, 3071, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3072, 3088, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 763, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 64, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 65, 344, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 345, 371, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 372, 494, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 495, 599, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 631, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 632, 742, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 743, 1023, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1024, 1128, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1129, 1173, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1174, 1199, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1301, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1302, 1582, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1583, 1636, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1637, 1732, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1733, 1949, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2014, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2015, 2294, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2295, 2321, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2322, 2399, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2649, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2847, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2848, 2879, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2880, 3159, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3160, 3309, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3310, 3449, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3593, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3594, 3729, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3730, 3743, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3744, 3799, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5108, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5109, 5111, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5112, 5179, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5372, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5373, 5379, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5778, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5779, 5781, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5782, 5849, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5932, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5933, 5999, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6149, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6342, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6343, 6449, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6479, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6480, 6493, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6494, 6599, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8071, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8072, 8182, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8183, 8463, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8464, 8568, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8569, 8613, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8614, 8689, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8782, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8783, 9039, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9302, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9303, 9452, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9453, 9659, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9731, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9732, 9769, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9866, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9867, 9869, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9958, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9959, 10108, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10109, 10359, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36253, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36254, 36349, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37798, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37799, 37850, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37851, 37910, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37911, 37948, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37949, 38229, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38230, 38249, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38312, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38313, 38433, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38434, 38462, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38463, 38649, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38816, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38817, 38962, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38963, 38966, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38967, 39246, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39247, 39396, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39397, 39649, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39728, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39729, 40008, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40009, 40044, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40045, 40142, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40143, 40158, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40159, 40438, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40439, 40490, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40491, 40550, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40551, 40588, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40589, 40869, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40870, 40987, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40988, 41019, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41020, 41299, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41300, 41449, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41450, 41589, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41666, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41667, 41947, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41948, 41999, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42000, 42097, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42098, 42377, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42378, 42401, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42402, 42497, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42498, 42527, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42528, 42807, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42808, 42844, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42845, 42907, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42908, 42957, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42958, 43238, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43239, 43342, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43343, 43388, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43389, 43589, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55388, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55389, 55492, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55493, 55538, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55539, 55818, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55819, 55968, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55969, 56249, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56250, 56399, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56400, 56679, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56680, 56739, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66500, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66501, 66780, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66781, 66807, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66808, 66930, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66931, 67211, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67212, 67305, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67306, 67335, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68796, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68797, 68809, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68810, 68913, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68914, 68935, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174540, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174541, 178360, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178361, 178780, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 186960, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186961, 190230, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190231, 191480, CAM_IMX374_SET_A_FULL_1046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191481, 191760, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191761, 191980, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 147760, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147761, 147820, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147821, 149200, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 162040, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162041, 164180, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 152540, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 152541, 156820, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156821, 157340, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157341, 160580, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 126700, CAM_IMX374_SET_A_FULL_860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126701, 126960, CAM_IMX374_SET_A_FULL_995_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126961, 129040, CAM_IMX374_SET_A_FULL_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129041, 130380, CAM_IMX374_SET_A_FULL_1046_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_A_mode[] = {
	{
		sensor_imx374_setfile_A_mipi_channel_FULL,	ARRAY_SIZE(sensor_imx374_setfile_A_mipi_channel_FULL),
		sensor_imx374_setfile_A_mipi_setting_FULL,	ARRAY_SIZE(sensor_imx374_setfile_A_mipi_setting_FULL)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
enum {
	CAM_IMX374_SET_B_4K2K_1196_MHZ = 0,
	CAM_IMX374_SET_B_4K2K_1011_MHZ = 1,
	CAM_IMX374_SET_B_4K2K_1139_MHZ = 2,
	CAM_IMX374_SET_B_4K2K_1094_MHZ = 3,
};

struct cam_sensor_i2c_reg_array MIPI_4K2K_1196_MHZ_REG_ARRAY[] = {
    {0x030F, 0x76, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_4K2K_1011_MHZ_REG_ARRAY[] = {
    {0x030F, 0x3C, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_4K2K_1139_MHZ_REG_ARRAY[] = {
    {0x030F, 0x64, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_4K2K_1094_MHZ_REG_ARRAY[] = {
    {0x030F, 0x56, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_B_mipi_4K2K_1196_mhz[] = {
    { MIPI_4K2K_1196_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_4K2K_1196_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_B_mipi_4K2K_1011_mhz[] = {
    { MIPI_4K2K_1011_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_4K2K_1011_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_B_mipi_4K2K_1139_mhz[] = {
    { MIPI_4K2K_1139_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_4K2K_1139_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx374_setfile_B_mipi_4K2K_1094_mhz[] = {
    { MIPI_4K2K_1094_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_4K2K_1094_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx374_setfile_B_mipi_setting_4K2K[] = {
	{ "1196 Mhz",
	  sensor_imx374_setfile_B_mipi_4K2K_1196_mhz, ARRAY_SIZE(sensor_imx374_setfile_B_mipi_4K2K_1196_mhz) },
	{ "1011 Mhz",
	  sensor_imx374_setfile_B_mipi_4K2K_1011_mhz, ARRAY_SIZE(sensor_imx374_setfile_B_mipi_4K2K_1011_mhz) },
	{ "1139 Mhz",
	  sensor_imx374_setfile_B_mipi_4K2K_1139_mhz, ARRAY_SIZE(sensor_imx374_setfile_B_mipi_4K2K_1139_mhz) },
	{ "1094 Mhz",
	  sensor_imx374_setfile_B_mipi_4K2K_1094_mhz, ARRAY_SIZE(sensor_imx374_setfile_B_mipi_4K2K_1094_mhz) },
};

static const struct cam_mipi_channel sensor_imx374_setfile_B_mipi_channel_4K2K[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10733, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10734, 10781, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10782, 10808, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10809, 10838, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9686, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9687, 9761, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9762, 9836, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9837, 9896, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9897, 9911, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9912, 9938, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1362, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1363, 1437, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1438, 1513, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1708, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1709, 1738, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4450, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4451, 4458, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2258, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2259, 2308, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2309, 2327, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2328, 2551, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2552, 2560, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2561, 2563, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 3049, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3050, 3065, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3066, 3088, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 367, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 368, 462, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 463, 517, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 518, 599, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 672, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 673, 822, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 823, 972, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 973, 1093, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1094, 1122, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1123, 1199, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1625, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1626, 1775, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1776, 1949, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2317, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2318, 2399, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2610, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2611, 2632, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2633, 2649, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2815, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2816, 2916, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2917, 2954, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2955, 3402, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3403, 3421, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3422, 3449, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3699, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3700, 3731, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3732, 3799, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5124, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5125, 5179, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5228, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5229, 5274, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5275, 5279, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5794, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5795, 5849, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5999, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6149, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6449, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6525, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6526, 6599, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8112, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8113, 8262, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8263, 8412, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8413, 8533, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8534, 8562, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8563, 8689, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 9000, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9001, 9022, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9023, 9039, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9334, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9335, 9484, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9485, 9659, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9745, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9746, 9769, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9869, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10284, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10285, 10359, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36349, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37760, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37761, 37856, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37857, 38249, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38523, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38524, 38587, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38588, 38649, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38912, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38913, 38954, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38955, 38982, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38983, 39062, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39063, 39510, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39511, 39590, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39591, 39648, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39649, 39649, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39747, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39748, 39894, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39895, 39897, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39898, 40046, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40047, 40196, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40197, 40346, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40347, 40400, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40401, 40496, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40497, 40955, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40956, 41056, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41057, 41094, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41095, 41542, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41543, 41561, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41562, 41589, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41623, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41624, 41643, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41644, 41691, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41692, 41744, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41745, 41923, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41924, 42045, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42046, 42073, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42074, 42222, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42223, 42372, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42373, 42820, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42821, 42907, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42908, 42970, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42971, 43419, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43420, 43549, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43550, 43562, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43563, 43569, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43570, 43589, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55569, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55570, 55699, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55700, 55712, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55713, 55719, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55720, 56167, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56168, 56218, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56219, 56317, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56318, 56739, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66803, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66804, 66898, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66899, 66953, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66954, 67335, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68923, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68924, 68935, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 177100, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177101, 178780, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 189080, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189081, 189720, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189721, 191260, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191261, 191980, CAM_IMX374_SET_B_4K2K_1094_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 164180, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 153180, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153181, 154060, CAM_IMX374_SET_B_4K2K_1139_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154061, 157980, CAM_IMX374_SET_B_4K2K_1011_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157981, 160580, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 129240, CAM_IMX374_SET_B_4K2K_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129241, 130380, CAM_IMX374_SET_B_4K2K_1139_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_B_mode[] = {
	{
		sensor_imx374_setfile_B_mipi_channel_4K2K,	ARRAY_SIZE(sensor_imx374_setfile_B_mipi_channel_4K2K),
		sensor_imx374_setfile_B_mipi_setting_4K2K,	ARRAY_SIZE(sensor_imx374_setfile_B_mipi_setting_4K2K)
	},
};

/*************************************/
/* ========== DUMMY ================ */
/*************************************/
enum {
	CAM_FRONT_SET_DUMMY_MHZ = 0,
};

struct cam_sensor_i2c_reg_array MIPI_FRONT_DUMMY_MHZ_REG_ARRAY[] = {
};

static const struct cam_sensor_i2c_reg_setting sensor_front_setfile_dummy_mhz[] = {
    { MIPI_FRONT_DUMMY_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FRONT_DUMMY_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_front_setfile_dummy_mipi_setting[] = {
	{ "DUMMY Mhz",
	  sensor_front_setfile_dummy_mhz, ARRAY_SIZE(sensor_front_setfile_dummy_mhz) },
};

static const struct cam_mipi_channel sensor_front_setfile_dummy_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_FRONT_SET_DUMMY_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_C_mode[] = {
	{
		sensor_front_setfile_dummy_channel,	ARRAY_SIZE(sensor_front_setfile_dummy_channel),
		sensor_front_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_front_setfile_dummy_mipi_setting)
	},
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_D_mode[] = {
	{
		sensor_front_setfile_dummy_channel,	ARRAY_SIZE(sensor_front_setfile_dummy_channel),
		sensor_front_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_front_setfile_dummy_mipi_setting)
	},
};
#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_IMX374_H_ */
