/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_

#include "cam_sensor_dev.h"

int num_tele_mipi_setting = 2;

/*
Mode A : Full mode, 2-Binning 120fps (1708.8 / 1660.8 / 1670.4)
Mode B : 2-Binning 240fps (1968 / 1900.8 / 1910.4)
*/

/*************************************/
/* ========== Mode A =============== */
/*************************************/

enum {
	CAM_IMX754_SET_A_FULL_1660_MHZ = 0,
	CAM_IMX754_SET_A_FULL_1670_MHZ = 1,
	CAM_IMX754_SET_A_FULL_1708_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_FULL_1660_MHZ_REG_ARRAY[] = {
    {0x030F, 0x07, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_FULL_1670_MHZ_REG_ARRAY[] = {
    {0x030F, 0x0A, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_FULL_1708_MHZ_REG_ARRAY[] = {
    {0x030F, 0x16, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_FULL_1660_mhz[] = {
    { MIPI_FULL_1660_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_1660_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_FULL_1670_mhz[] = {
    { MIPI_FULL_1670_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_1670_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_FULL_1708_mhz[] = {
    { MIPI_FULL_1708_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FULL_1708_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx754_setfile_A_mipi_setting_FULL[] = {
	{ "1660 Mhz",
	  sensor_imx754_setfile_A_mipi_FULL_1660_mhz, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_FULL_1660_mhz) },
	{ "1670 Mhz",
	  sensor_imx754_setfile_A_mipi_FULL_1670_mhz, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_FULL_1670_mhz) },
	{ "1708 Mhz",
	  sensor_imx754_setfile_A_mipi_FULL_1708_mhz, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_FULL_1708_mhz) },
};

static const struct cam_mipi_channel sensor_imx754_setfile_A_mipi_channel_FULL[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10588, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10589, 10757, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10758, 10820, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10821, 10829, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10830, 10838, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9808, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9809, 9874, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9875, 9938, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1221, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1222, 1285, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1286, 1429, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1430, 1483, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1484, 1513, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1563, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1564, 1732, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1733, 1738, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4377, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4378, 4426, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4427, 4458, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2255, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2256, 2330, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2331, 2492, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2493, 2563, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2964, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2965, 3088, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 751, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 752, 763, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 76, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 77, 415, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 416, 540, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 541, 558, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 559, 599, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 917, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 918, 1049, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1050, 1199, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1343, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1344, 1471, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1472, 1758, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1759, 1866, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1867, 1949, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2026, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2027, 2365, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2366, 2399, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2413, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2414, 2464, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2465, 2563, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2564, 2649, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2810, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2811, 2960, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2961, 3285, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3286, 3435, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3436, 3449, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3528, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3529, 3799, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5042, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5043, 5149, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5150, 5179, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5819, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5820, 5849, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5953, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5954, 5999, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6004, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6005, 6103, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6104, 6149, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6449, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6599, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8357, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8358, 8489, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8490, 8689, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8803, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8804, 8854, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8855, 8953, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8954, 9039, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9384, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9385, 9429, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9430, 9534, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9535, 9659, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9769, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9826, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9827, 9869, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10006, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10007, 10272, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10273, 10359, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36325, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36326, 36349, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37836, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37837, 37985, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37986, 37986, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37987, 38249, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38592, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38593, 38649, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38692, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38693, 38721, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38722, 38738, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38739, 39063, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39064, 39206, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39207, 39537, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39538, 39649, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39670, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39671, 39676, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39677, 40001, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40002, 40148, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40149, 40151, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40152, 40476, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40477, 40625, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40626, 40626, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40627, 40950, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40951, 41100, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41101, 41425, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41426, 41575, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41576, 41589, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41592, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41593, 41917, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41918, 42067, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42068, 42391, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42392, 42541, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42542, 42807, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42808, 43014, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43015, 43340, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43341, 43490, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43491, 43589, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55490, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55491, 55640, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55641, 55965, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55966, 56115, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56116, 56439, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56440, 56589, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56590, 56739, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66512, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66513, 66851, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66852, 66976, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66977, 66994, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66995, 67325, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67326, 67335, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68604, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68605, 68718, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68719, 68746, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68747, 68751, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68752, 68840, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68841, 68935, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174180, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174181, 177940, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177941, 178780, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187420, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187421, 191980, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158830, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158831, 163660, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163661, 164180, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 154200, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154201, 155100, CAM_IMX754_SET_A_FULL_1670_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155101, 158830, CAM_IMX754_SET_A_FULL_1708_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158831, 160580, CAM_IMX754_SET_A_FULL_1660_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 130380, CAM_IMX754_SET_A_FULL_1708_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_A_mode[] = {
	{
		sensor_imx754_setfile_A_mipi_channel_FULL,	ARRAY_SIZE(sensor_imx754_setfile_A_mipi_channel_FULL),
		sensor_imx754_setfile_A_mipi_setting_FULL,	ARRAY_SIZE(sensor_imx754_setfile_A_mipi_setting_FULL)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
enum {
	CAM_IMX754_SET_B_2BIN240_1900_MHZ = 0,
	CAM_IMX754_SET_B_2BIN240_1910_MHZ = 1,
	CAM_IMX754_SET_B_2BIN240_1968_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_2BIN240_1900_MHZ_REG_ARRAY[] = {
    {0x030F, 0x52, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_2BIN240_1910_MHZ_REG_ARRAY[] = {
    {0x030F, 0x55, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_2BIN240_1968_MHZ_REG_ARRAY[] = {
    {0x030F, 0x67, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_2BIN240_1900_mhz[] = {
    { MIPI_2BIN240_1900_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_2BIN240_1900_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_2BIN240_1910_mhz[] = {
    { MIPI_2BIN240_1910_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_2BIN240_1910_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_2BIN240_1968_mhz[] = {
    { MIPI_2BIN240_1968_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_2BIN240_1968_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx754_setfile_B_mipi_setting_2BIN240[] = {
	{ "1900 Mhz",
	  sensor_imx754_setfile_B_mipi_2BIN240_1900_mhz, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_2BIN240_1900_mhz) },
	{ "1910 Mhz",
	  sensor_imx754_setfile_B_mipi_2BIN240_1910_mhz, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_2BIN240_1910_mhz) },
	{ "1968 Mhz",
	  sensor_imx754_setfile_B_mipi_2BIN240_1968_mhz, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_2BIN240_1968_mhz) },
};

static const struct cam_mipi_channel sensor_imx754_setfile_B_mipi_channel_2BIN240[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10654, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10655, 10708, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10709, 10763, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10764, 10838, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9873, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9874, 9923, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9924, 9938, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1184, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1185, 1245, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1246, 1455, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1456, 1503, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1504, 1513, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1629, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1630, 1683, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1684, 1738, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4442, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4443, 4458, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2257, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2258, 2296, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2297, 2323, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2324, 2332, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2333, 2528, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2529, 2563, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 3014, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3015, 3038, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3039, 3088, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 763, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 208, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 209, 316, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 317, 426, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 427, 599, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 601, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 602, 639, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 640, 1047, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1048, 1147, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1148, 1192, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1193, 1199, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1268, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1269, 1389, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1390, 1811, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1812, 1906, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1907, 1949, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2158, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2159, 2266, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2267, 2376, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2377, 2399, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2595, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2596, 2641, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2642, 2649, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2814, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2815, 2892, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2893, 2947, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2948, 2964, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2965, 3357, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3358, 3449, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3628, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3629, 3676, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3677, 3778, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3779, 3799, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5013, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5014, 5075, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5076, 5179, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5999, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6135, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6136, 6149, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6189, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6190, 6449, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6500, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6501, 6599, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8041, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8042, 8079, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8080, 8487, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8488, 8587, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8588, 8632, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8633, 8689, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8985, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8986, 9031, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9032, 9039, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9429, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9430, 9469, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9470, 9579, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9580, 9659, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9692, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9693, 9746, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9747, 9769, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9819, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9820, 9869, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10259, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10260, 10359, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36349, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37771, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37772, 37901, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37902, 37921, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37922, 38110, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38111, 38249, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38382, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38383, 38478, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38479, 38532, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38533, 38649, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38656, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38657, 38772, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38773, 38806, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38807, 39199, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39200, 39318, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39319, 39478, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39479, 39649, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39868, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39869, 39996, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39997, 40018, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40019, 40411, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40412, 40541, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40542, 40561, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40562, 40750, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40751, 40891, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40892, 40954, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40955, 41032, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41033, 41087, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41088, 41104, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41105, 41497, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41498, 41589, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41607, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41608, 42000, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42001, 42150, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42151, 42544, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42545, 42694, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42695, 43087, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43088, 43286, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43287, 43589, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55436, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55437, 55780, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55781, 55930, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55931, 56323, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56324, 56473, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56474, 56739, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66644, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66645, 66752, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66753, 66862, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66863, 67255, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67256, 67335, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68586, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68587, 68618, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68619, 68736, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68737, 68935, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 176800, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176801, 177720, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177721, 178780, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187660, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187661, 188620, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188621, 191980, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 159880, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159881, 164180, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 155080, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155081, 155880, CAM_IMX754_SET_B_2BIN240_1910_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155881, 159880, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159881, 160580, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 127100, CAM_IMX754_SET_B_2BIN240_1968_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 127101, 130380, CAM_IMX754_SET_B_2BIN240_1900_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_B_mode[] = {
	{
		sensor_imx754_setfile_B_mipi_channel_2BIN240,	ARRAY_SIZE(sensor_imx754_setfile_B_mipi_channel_2BIN240),
		sensor_imx754_setfile_B_mipi_setting_2BIN240,	ARRAY_SIZE(sensor_imx754_setfile_B_mipi_setting_2BIN240)
	},
};

/*************************************/
/* ========== DUMMY ================ */
/*************************************/
enum {
	CAM_TELE_SET_DUMMY_MHZ = 0,
};

struct cam_sensor_i2c_reg_array MIPI_TELE_DUMMY_MHZ_REG_ARRAY[] = {
};

static const struct cam_sensor_i2c_reg_setting sensor_tele_setfile_dummy_mhz[] = {
    { MIPI_TELE_DUMMY_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_TELE_DUMMY_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_tele_setfile_dummy_mipi_setting[] = {
	{ "DUMMY Mhz",
	  sensor_tele_setfile_dummy_mhz, ARRAY_SIZE(sensor_tele_setfile_dummy_mhz) },
};

static const struct cam_mipi_channel sensor_tele_setfile_dummy_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_TELE_SET_DUMMY_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_C_mode[] = {
	{
		sensor_tele_setfile_dummy_channel,	ARRAY_SIZE(sensor_tele_setfile_dummy_channel),
		sensor_tele_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_tele_setfile_dummy_mipi_setting)
	},
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_D_mode[] = {
	{
		sensor_tele_setfile_dummy_channel,	ARRAY_SIZE(sensor_tele_setfile_dummy_channel),
		sensor_tele_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_tele_setfile_dummy_mipi_setting)
	},
};
#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_ */
