/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_S5K3LU_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_S5K3LU_H_

#include "cam_sensor_dev.h"

int num_front_mipi_setting = 2;

/*
3LU_220830
Mode A : no-bin (10/12bit) (1862.4 / 1849.6 / 1811.2)
Mode B : FastAE/SM 2x2 bin (10bit) (1164.8 / 1062.4 / 1116.8 / 1196.8)
Mode C : Min power 2x2 bin + skipping (10bit) (478.4 / 462.4 / 451.2 / 419.2)
*/

/*************************************/
/* ========== Mode A =============== */
/*************************************/

enum {
	CAM_S5K3LU_SET_A_1862_MHZ = 0,
	CAM_S5K3LU_SET_A_1849_MHZ = 1,
	CAM_S5K3LU_SET_A_1811_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_SET_A_1862_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x0123, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_SET_A_1849_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x0121, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_SET_A_1811_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x011B, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_A_mipi_1862_mhz[] = {
    { MIPI_SET_A_1862_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_A_1862_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_A_mipi_1849_mhz[] = {
    { MIPI_SET_A_1849_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_A_1849_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_A_mipi_1811_mhz[] = {
    { MIPI_SET_A_1811_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_A_1811_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5k3lu_setfile_A_mipi_setting[] = {
	{ "1862 Msps",
	  sensor_s5k3lu_setfile_A_mipi_1862_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_A_mipi_1862_mhz) },
	{ "1849 Msps",
	  sensor_s5k3lu_setfile_A_mipi_1849_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_A_mipi_1849_mhz) },
	{ "1811 Msps",
	  sensor_s5k3lu_setfile_A_mipi_1811_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_A_mipi_1811_mhz) },
};

static const struct cam_mipi_channel sensor_s5k3lu_setfile_A_mipi_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10600, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10601, 10745, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10746, 10754, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10755, 10815, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10816, 10838, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9695, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9696, 9737, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9738, 9775, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9776, 9845, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9846, 9883, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9884, 9938, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1184, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1185, 1250, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1251, 1308, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1309, 1361, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1362, 1404, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1405, 1431, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1432, 1513, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1575, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1576, 1720, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1721, 1729, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1730, 1738, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4399, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4400, 4438, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4439, 4458, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4399, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4400, 4413, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2242, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2243, 2261, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2262, 2326, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2327, 2366, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2367, 2458, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2459, 2521, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2522, 2528, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2529, 2563, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2958, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2959, 2990, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2991, 3088, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 724, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 725, 763, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 100, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 101, 391, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 392, 408, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 409, 531, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 532, 599, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 691, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 692, 775, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 776, 851, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 852, 991, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 992, 1066, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1067, 1186, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1187, 1199, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1268, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1269, 1400, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1401, 1517, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1518, 1623, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1624, 1709, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1710, 1763, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1764, 1946, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1947, 1949, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2050, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2051, 2341, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2342, 2358, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2359, 2399, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2508, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2509, 2587, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2588, 2648, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2649, 2649, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2785, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2786, 2822, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2823, 2953, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2954, 3033, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3034, 3216, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3217, 3343, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3344, 3356, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3357, 3449, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3517, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3518, 3581, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3582, 3799, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5179, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5188, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5189, 5236, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5237, 5279, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5308, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5309, 5379, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5855, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5856, 5999, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6048, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6049, 6127, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6128, 6149, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6151, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6152, 6236, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6237, 6291, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6292, 6449, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6497, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6498, 6524, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6525, 6599, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8131, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8132, 8215, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8216, 8291, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8292, 8431, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8432, 8506, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8507, 8626, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8627, 8646, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8647, 8689, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8705, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8706, 8898, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8899, 8977, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8978, 9038, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9039, 9039, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9238, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9239, 9319, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9320, 9364, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9365, 9459, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9460, 9541, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9542, 9626, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9627, 9659, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9769, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9869, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9963, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9964, 10052, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10053, 10213, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10214, 10359, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36322, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36323, 36349, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37778, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37779, 37854, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37855, 37962, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37963, 37994, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37995, 38249, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38316, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38317, 38452, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38453, 38494, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38495, 38649, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38866, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38867, 38960, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38961, 39060, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39061, 39114, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39115, 39298, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39299, 39360, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39361, 39438, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39439, 39649, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39721, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39722, 39771, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39772, 40063, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40064, 40203, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40204, 40290, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40291, 40418, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40419, 40494, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40495, 40602, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40603, 40634, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40635, 40925, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40926, 40962, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40963, 41093, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41094, 41173, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41174, 41356, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41357, 41483, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41484, 41496, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41497, 41589, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41664, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41665, 41804, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41805, 42107, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42108, 42200, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42201, 42241, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42242, 42440, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42441, 42450, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42451, 42690, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42691, 43052, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43053, 43134, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43135, 43216, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43217, 43321, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43322, 43356, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43357, 43437, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43438, 43577, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43578, 43589, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55284, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55285, 55366, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55367, 55471, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55472, 55506, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55507, 55587, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55588, 55727, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55728, 56031, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56032, 56221, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56222, 56282, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56283, 56474, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56475, 56648, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56649, 56739, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66536, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66537, 66827, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66828, 66844, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66845, 66967, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66968, 67151, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67152, 67285, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67286, 67335, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68651, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68652, 68693, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68694, 68935, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174960, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174961, 178540, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178541, 178780, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187340, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187341, 188620, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188621, 191980, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 160920, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 160921, 162020, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162021, 164180, CAM_S5K3LU_SET_A_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 160580, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 125700, CAM_S5K3LU_SET_A_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125701, 126540, CAM_S5K3LU_SET_A_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126541, 130380, CAM_S5K3LU_SET_A_1862_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_A_mode[] = {
	{
		sensor_s5k3lu_setfile_A_mipi_channel,	ARRAY_SIZE(sensor_s5k3lu_setfile_A_mipi_channel),
		sensor_s5k3lu_setfile_A_mipi_setting,	ARRAY_SIZE(sensor_s5k3lu_setfile_A_mipi_setting)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
enum {
	CAM_S5K3LU_SET_B_1164_MHZ = 0,
	CAM_S5K3LU_SET_B_1062_MHZ = 1,
	CAM_S5K3LU_SET_B_1116_MHZ = 2,
	CAM_S5K3LU_SET_B_1196_MHZ = 3,
};

struct cam_sensor_i2c_reg_array MIPI_SET_B_1164_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x00B6, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_SET_B_1062_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x014C, 0x00, 0x00},
	{0x0312, 0x0002, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_SET_B_1116_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x015D, 0x00, 0x00},
	{0x0312, 0x0002, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_SET_B_1196_MHZ_REG_ARRAY[] = {
    {0x030E, 0x0003, 0x00, 0x00},
	{0x0310, 0x00BB, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_B_mipi_1164_mhz[] = {
    { MIPI_SET_B_1164_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_B_1164_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_B_mipi_1062_mhz[] = {
    { MIPI_SET_B_1062_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_B_1062_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_B_mipi_1116_mhz[] = {
    { MIPI_SET_B_1116_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_B_1116_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5k3lu_setfile_B_mipi_1196_mhz[] = {
    { MIPI_SET_B_1196_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_SET_B_1196_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5k3lu_setfile_B_mipi_setting[] = {
	{ "1164 Msps",
	  sensor_s5k3lu_setfile_B_mipi_1164_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_1164_mhz) },
	{ "1062 Msps",
	  sensor_s5k3lu_setfile_B_mipi_1062_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_1062_mhz) },
	{ "1116 Msps",
	  sensor_s5k3lu_setfile_B_mipi_1116_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_1116_mhz) },
	{ "1196 Msps",
	  sensor_s5k3lu_setfile_B_mipi_1196_mhz, ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_1196_mhz) },
};

static const struct cam_mipi_channel sensor_s5k3lu_setfile_B_mipi_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10573, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10574, 10642, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10643, 10654, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10655, 10712, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10713, 10780, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10781, 10785, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10786, 10838, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9671, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9672, 9713, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9714, 9740, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9741, 9741, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9742, 9810, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9811, 9848, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9849, 9873, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9874, 9880, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9881, 9938, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1173, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1174, 1241, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1242, 1265, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1266, 1311, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1312, 1380, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1381, 1392, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1393, 1449, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1450, 1450, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1451, 1513, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1548, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1549, 1617, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1618, 1629, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1630, 1687, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1688, 1738, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4412, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4413, 4458, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2265, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2266, 2313, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2314, 2333, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2334, 2406, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2407, 2439, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2440, 2471, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2472, 2546, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2547, 2563, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2984, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2985, 3014, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3015, 3049, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3050, 3088, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 737, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 738, 763, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 47, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 48, 184, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 185, 209, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 210, 324, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 325, 461, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 462, 470, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 471, 583, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 584, 599, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 643, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 644, 727, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 728, 781, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 782, 783, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 784, 920, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 921, 996, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 997, 1046, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1047, 1060, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1061, 1197, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1198, 1199, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1246, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1247, 1383, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1384, 1430, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1431, 1523, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1524, 1661, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1662, 1685, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1686, 1798, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1799, 1801, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1802, 1938, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1939, 1949, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 1997, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1998, 2134, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2135, 2159, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2160, 2274, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2275, 2399, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2534, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2535, 2633, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2634, 2649, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2831, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2832, 2927, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2928, 2966, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2967, 3112, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3113, 3179, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3180, 3243, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3244, 3393, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3394, 3432, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3433, 3449, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3568, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3569, 3629, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3630, 3699, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3700, 3799, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5137, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5138, 5179, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5238, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5239, 5277, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5278, 5279, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5807, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5808, 5849, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5917, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5918, 5999, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6074, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6075, 6149, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6212, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6213, 6276, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6277, 6352, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6353, 6449, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6451, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6452, 6485, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6486, 6536, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6537, 6599, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8083, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8084, 8167, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8168, 8221, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8222, 8223, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8224, 8360, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8361, 8436, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8437, 8486, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8487, 8500, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8501, 8637, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8638, 8689, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8767, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8768, 8924, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8925, 9023, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9024, 9039, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9325, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9326, 9401, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9402, 9411, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9412, 9465, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9466, 9602, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9603, 9659, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9755, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9756, 9769, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9773, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9774, 9864, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9865, 9869, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10028, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10029, 10094, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10095, 10141, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10142, 10168, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10169, 10305, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10306, 10359, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36349, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37771, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37772, 37781, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37782, 37911, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37912, 38050, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38051, 38174, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38175, 38189, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38190, 38249, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38351, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38352, 38378, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38379, 38515, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38516, 38608, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38609, 38649, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38726, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38727, 38875, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38876, 38985, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38986, 39015, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39016, 39153, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39154, 39244, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39245, 39293, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39294, 39430, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39431, 39497, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39498, 39570, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39571, 39649, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39662, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39663, 39707, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39708, 39857, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39858, 39915, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39916, 39989, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39990, 40134, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40135, 40168, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40169, 40270, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40271, 40411, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40412, 40421, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40422, 40551, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40552, 40690, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40691, 40814, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40815, 40829, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40830, 40971, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40972, 41067, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41068, 41106, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41107, 41252, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41253, 41319, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41320, 41383, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41384, 41533, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41534, 41572, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41573, 41589, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41632, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41633, 41668, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41669, 41695, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41696, 41772, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41773, 41909, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41910, 41921, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41922, 41961, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41962, 42049, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42050, 42186, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42187, 42227, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42228, 42314, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42315, 42326, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42327, 42464, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42465, 42493, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42494, 42567, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42568, 42604, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42605, 42741, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42742, 42759, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42760, 42820, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42821, 42881, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42882, 43018, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43019, 43025, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43026, 43073, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43074, 43158, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43159, 43296, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43297, 43326, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43327, 43436, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43437, 43573, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43574, 43579, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43580, 43589, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55308, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55309, 55446, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55447, 55476, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55477, 55586, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55587, 55723, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55724, 55729, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55730, 55842, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55843, 55861, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55862, 56000, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56001, 56095, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56096, 56098, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56099, 56140, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56141, 56278, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56279, 56348, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56349, 56364, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56365, 56418, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56419, 56555, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56556, 56601, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56602, 56630, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56631, 56695, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56696, 56739, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66483, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66484, 66620, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66621, 66645, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66646, 66760, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66761, 66897, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66898, 66906, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66907, 67019, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67020, 67037, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67038, 67175, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67176, 67272, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67273, 67315, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67316, 67335, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68587, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68588, 68724, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68725, 68741, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68742, 68809, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68810, 68864, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68865, 68935, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174300, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174301, 176480, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176481, 178780, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 185420, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185421, 187350, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187351, 189310, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189311, 190150, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190151, 190980, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190981, 191980, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 146550, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 146551, 148220, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 148221, 149200, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158830, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158831, 160710, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 160711, 162460, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162461, 163050, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163051, 164180, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 151980, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151981, 153530, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153531, 155340, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155341, 156620, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156621, 157140, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157141, 157430, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157431, 158830, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158831, 160580, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 124240, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124241, 125160, CAM_S5K3LU_SET_B_1164_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125161, 126490, CAM_S5K3LU_SET_B_1116_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126491, 128760, CAM_S5K3LU_SET_B_1196_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 128761, 129540, CAM_S5K3LU_SET_B_1062_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129541, 130380, CAM_S5K3LU_SET_B_1164_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_B_mode[] = {
	{
		sensor_s5k3lu_setfile_B_mipi_channel,	ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_channel),
		sensor_s5k3lu_setfile_B_mipi_setting,	ARRAY_SIZE(sensor_s5k3lu_setfile_B_mipi_setting)
	},
};

/*************************************/
/* ========== DUMMY ================ */
/*************************************/
enum {
	CAM_FRONT_SET_DUMMY_MHZ = 0,
};

struct cam_sensor_i2c_reg_array MIPI_FRONT_DUMMY_MHZ_REG_ARRAY[] = {
};

static const struct cam_sensor_i2c_reg_setting sensor_front_setfile_dummy_mhz[] = {
    { MIPI_FRONT_DUMMY_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_FRONT_DUMMY_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_front_setfile_dummy_mipi_setting[] = {
	{ "DUMMY Msps",
	  sensor_front_setfile_dummy_mhz, ARRAY_SIZE(sensor_front_setfile_dummy_mhz) },
};

static const struct cam_mipi_channel sensor_front_setfile_dummy_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_FRONT_SET_DUMMY_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_C_mode[] = {
	{
		sensor_front_setfile_dummy_channel,	ARRAY_SIZE(sensor_front_setfile_dummy_channel),
		sensor_front_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_front_setfile_dummy_mipi_setting)
	},
};

static const struct cam_mipi_sensor_mode sensor_front_mipi_D_mode[] = {
	{
		sensor_front_setfile_dummy_channel,	ARRAY_SIZE(sensor_front_setfile_dummy_channel),
		sensor_front_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_front_setfile_dummy_mipi_setting)
	},
};

#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_S5K3LU_H_ */
