/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_S5KGN3_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_S5KGN3_H_

#include "cam_sensor_dev.h"

int num_wide_mipi_setting = 3;

/*
SET A : Full-Remosaic, 4SUM (10Bit)
SET B : 4SUM+A2A2 (10Bit), 8k (10Bit)
SET C : Full-Remosaic, 4SUM (12Bit)
*/

/*************************************/
/* ========== Mode A =============== */
/*************************************/

enum {
	CAM_S5KGN3_SET_A_3046_MHZ = 0,
	CAM_S5KGN3_SET_A_3008_MHZ = 1,
	CAM_S5KGN3_SET_A_2982_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_A_3046_MHZ_REG_ARRAY[] = {
    {0x0310, 0x00EE, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_A_3008_MHZ_REG_ARRAY[] = {
    {0x0310, 0x00EB, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_A_2982_MHZ_REG_ARRAY[] = {
    {0x0310, 0x00E9, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_A_3046_mhz[] = {
    { MIPI_A_3046_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_A_3046_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_A_3008_mhz[] = {
    { MIPI_A_3008_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_A_3008_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_A_2982_mhz[] = {
    { MIPI_A_2982_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_A_2982_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5kgn3_setfile_A_mipi_setting[] = {
	{ "3046 Mhz",
	  sensor_s5kgn3_setfile_A_3046_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_A_3046_mhz) },
	{ "3008 Mhz",
	  sensor_s5kgn3_setfile_A_3008_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_A_3008_mhz) },
	{ "2982 Mhz",
	  sensor_s5kgn3_setfile_A_2982_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_A_2982_mhz) },
};

static const struct cam_mipi_channel sensor_s5kgn3_setfile_A_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10626, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10627, 10635, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10636, 10692, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10693, 10771, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10772, 10778, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10779, 10838, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9683, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9684, 9756, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9757, 9776, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9777, 9828, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9829, 9901, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9902, 9919, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9920, 9938, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1184, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1185, 1227, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1228, 1300, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1301, 1328, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1329, 1372, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1373, 1445, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1446, 1471, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1472, 1513, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1601, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1602, 1610, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1611, 1668, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1669, 1738, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4388, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4389, 4404, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4405, 4458, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2289, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2290, 2362, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2363, 2389, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2390, 2434, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2435, 2507, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2508, 2532, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2533, 2563, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2978, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2979, 2991, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2992, 3051, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3052, 3088, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 729, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 730, 763, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 152, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 153, 170, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 171, 285, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 286, 442, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 443, 457, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 458, 572, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 573, 599, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 666, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 667, 811, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 812, 851, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 852, 956, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 957, 1101, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1102, 1138, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1139, 1199, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1210, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1211, 1269, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1270, 1355, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1356, 1500, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1501, 1556, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1557, 1645, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1646, 1791, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1792, 1842, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1843, 1936, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1937, 1949, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2102, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2103, 2120, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2121, 2235, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2236, 2392, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2393, 2399, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2486, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2487, 2519, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2520, 2631, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2632, 2649, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2793, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2794, 2879, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2880, 3024, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3025, 3078, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3079, 3169, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3170, 3314, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3315, 3364, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3365, 3449, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3556, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3557, 3582, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3583, 3701, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3702, 3799, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5045, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5046, 5096, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5097, 5179, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5190, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5191, 5279, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5315, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5316, 5363, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5364, 5379, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5766, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5767, 5849, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5881, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5882, 5999, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6026, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6027, 6059, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6060, 6149, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6189, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6190, 6291, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6292, 6436, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6437, 6449, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6467, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6468, 6505, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6506, 6599, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8106, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8107, 8251, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8252, 8291, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8292, 8396, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8397, 8541, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8542, 8578, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8579, 8686, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8687, 8689, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8731, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8732, 8876, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8877, 8909, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8910, 9021, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9022, 9039, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9245, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9246, 9293, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9294, 9391, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9392, 9536, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9537, 9579, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9580, 9659, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9670, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9671, 9769, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9843, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9844, 9869, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9979, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9980, 10090, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10091, 10124, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10125, 10269, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10270, 10359, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36224, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36225, 36336, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36337, 36349, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37799, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37800, 37944, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37945, 38008, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38009, 38089, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38090, 38234, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38235, 38249, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38381, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38382, 38429, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38430, 38526, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38527, 38649, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38755, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38756, 38788, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38789, 38933, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38934, 39041, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39042, 39078, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39079, 39223, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39224, 39326, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39327, 39368, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39369, 39513, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39514, 39611, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39612, 39649, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39713, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39714, 39792, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39793, 39859, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39860, 40004, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40005, 40077, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40078, 40149, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40150, 40294, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40295, 40363, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40364, 40439, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40440, 40584, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40585, 40648, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40649, 40729, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40730, 40874, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40875, 40933, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40934, 41019, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41020, 41164, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41165, 41218, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41219, 41309, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41310, 41454, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41455, 41504, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41505, 41589, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41608, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41609, 41609, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41610, 41753, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41754, 41898, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41899, 42041, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42042, 42188, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42189, 42329, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42330, 42478, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42479, 42617, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42618, 42768, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42769, 42906, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42907, 43058, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43059, 43194, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43195, 43348, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43349, 43483, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43484, 43589, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55344, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55345, 55498, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55499, 55633, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55634, 55789, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55790, 55921, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55922, 56079, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56080, 56209, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56210, 56369, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56370, 56497, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56498, 56659, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56660, 56739, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66588, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66589, 66606, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66607, 66721, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66722, 66878, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66879, 66893, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66894, 67008, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67009, 67168, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67169, 67179, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67180, 67295, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67296, 67335, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68646, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68647, 68726, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68727, 68874, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68875, 68933, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68934, 68935, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 175520, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 175521, 176180, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176181, 178420, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178421, 178780, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187120, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187121, 187640, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187641, 190020, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190021, 191980, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 146500, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 146501, 147520, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147521, 149200, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158980, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158981, 161020, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 161021, 163920, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163921, 164180, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 152300, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 152301, 153260, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153261, 155220, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155221, 158120, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158121, 158980, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158981, 160580, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 124600, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124601, 126200, CAM_S5KGN3_SET_A_3008_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126201, 129160, CAM_S5KGN3_SET_A_3046_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129161, 130340, CAM_S5KGN3_SET_A_2982_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 130341, 130380, CAM_S5KGN3_SET_A_3008_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_A_mode[] = {
	{
		sensor_s5kgn3_setfile_A_channel,	ARRAY_SIZE(sensor_s5kgn3_setfile_A_channel),
		sensor_s5kgn3_setfile_A_mipi_setting,	ARRAY_SIZE(sensor_s5kgn3_setfile_A_mipi_setting)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
enum {
	CAM_S5KGN3_SET_B_2585_MHZ = 0,
	CAM_S5KGN3_SET_B_2547_MHZ = 1,
	CAM_S5KGN3_SET_B_2483_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_B_2585_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x00CA, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_B_2547_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x00C7, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_B_2483_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x00C2, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_B_2585_mhz[] = {
    { MIPI_B_2585_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_B_2585_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_B_2547_mhz[] = {
    { MIPI_B_2547_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_B_2547_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_B_2483_mhz[] = {
    { MIPI_B_2483_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_B_2483_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5kgn3_setfile_B_mipi_setting[] = {
	{ "2585 Mhz",
	  sensor_s5kgn3_setfile_B_2585_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_B_2585_mhz) },
	{ "2547 Mhz",
	  sensor_s5kgn3_setfile_B_2547_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_B_2547_mhz) },
	{ "2483 Mhz",
	  sensor_s5kgn3_setfile_B_2483_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_B_2483_mhz) },
};

static const struct cam_mipi_channel sensor_s5kgn3_setfile_B_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10619, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10620, 10643, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10644, 10680, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10681, 10742, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10743, 10764, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10765, 10804, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10805, 10838, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9673, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9674, 9696, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9697, 9772, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9773, 9795, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9796, 9819, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9820, 9892, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9893, 9916, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9917, 9938, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1205, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1206, 1290, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1291, 1328, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1329, 1412, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1413, 1451, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1452, 1513, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1594, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1595, 1619, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1620, 1656, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1657, 1718, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1719, 1738, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4396, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4397, 4401, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4402, 4458, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2255, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2256, 2268, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2269, 2330, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2331, 2376, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2377, 2391, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2392, 2453, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2454, 2497, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2498, 2514, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2515, 2563, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2945, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2946, 3009, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3010, 3066, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3067, 3088, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 721, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 722, 726, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 727, 763, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 15, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 16, 138, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 139, 187, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 188, 261, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 262, 385, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 386, 429, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 430, 508, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 509, 599, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 646, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 647, 691, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 692, 842, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 843, 889, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 890, 938, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 939, 1084, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1085, 1131, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1132, 1184, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1185, 1199, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1310, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1311, 1480, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1481, 1557, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1558, 1724, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1725, 1803, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1804, 1949, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 1965, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1966, 2088, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2089, 2137, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2138, 2211, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2212, 2335, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2336, 2379, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2380, 2399, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2503, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2504, 2513, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2514, 2636, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2637, 2649, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2810, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2811, 2836, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2837, 2960, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2961, 3053, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3054, 3083, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3084, 3206, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3207, 3295, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3296, 3329, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3330, 3449, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3479, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3480, 3489, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3490, 3618, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3619, 3732, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3733, 3799, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5045, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5046, 5179, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5930, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5931, 5999, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6043, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6044, 6053, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6054, 6149, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6183, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6184, 6304, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6305, 6427, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6428, 6449, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6573, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6574, 6592, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6593, 6599, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8086, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8087, 8131, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8132, 8282, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8283, 8329, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8330, 8378, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8379, 8524, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8525, 8571, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8572, 8624, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8625, 8689, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8780, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8781, 8893, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8894, 8903, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8904, 9026, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9027, 9039, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9329, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9330, 9448, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9449, 9573, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9574, 9659, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9700, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9701, 9769, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9848, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9849, 9869, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9990, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9991, 10016, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10017, 10113, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10114, 10236, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10237, 10258, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10259, 10359, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36230, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36231, 36349, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37825, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37826, 37917, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37918, 38067, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38068, 38090, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38091, 38213, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38214, 38249, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38311, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38312, 38349, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38350, 38509, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38510, 38553, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38554, 38595, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38596, 38649, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38735, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38736, 38858, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38859, 38878, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38879, 38981, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38982, 39105, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39106, 39120, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39121, 39228, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39229, 39351, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39352, 39361, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39362, 39474, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39475, 39601, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39602, 39649, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39737, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39738, 39745, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39746, 39868, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39869, 39980, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39981, 39991, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39992, 40144, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40145, 40238, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40239, 40361, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40362, 40465, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40466, 40557, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40558, 40707, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40708, 40730, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40731, 40853, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40854, 40950, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40951, 40976, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40977, 41100, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41101, 41193, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41194, 41223, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41224, 41346, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41347, 41435, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41436, 41469, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41470, 41589, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41633, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41634, 41719, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41720, 41756, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41757, 41879, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41880, 41959, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41960, 42003, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42004, 42126, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42127, 42198, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42199, 42249, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42250, 42372, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42373, 42438, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42439, 42495, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42496, 42618, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42619, 42677, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42678, 42741, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42742, 42864, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42865, 42917, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42918, 42988, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42989, 43111, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43112, 43157, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43158, 43234, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43235, 43357, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43358, 43396, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43397, 43480, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43481, 43589, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55261, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55262, 55307, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55308, 55384, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55385, 55507, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55508, 55546, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55547, 55630, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55631, 55789, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55790, 55876, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55877, 56033, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56034, 56123, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56124, 56278, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56279, 56369, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56370, 56523, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56524, 56615, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56616, 56739, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66451, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66452, 66574, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66575, 66623, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66624, 66697, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66698, 66821, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66822, 66865, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66866, 66944, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66945, 67067, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67068, 67108, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67109, 67190, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67191, 67313, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67314, 67335, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68648, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68649, 68756, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68757, 68892, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68893, 68935, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 175860, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 175861, 176060, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176061, 178520, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178521, 178780, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 185580, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185581, 185770, CAM_S5KGN3_SET_B_2483_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185771, 188360, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188361, 190630, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190631, 191980, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 146500, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 146501, 149070, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 149071, 149200, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158850, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158851, 161280, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 161281, 163740, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163741, 164180, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 153970, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153971, 156360, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156361, 158850, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158851, 160580, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 124630, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124631, 126800, CAM_S5KGN3_SET_B_2547_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126801, 129520, CAM_S5KGN3_SET_B_2585_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129521, 130380, CAM_S5KGN3_SET_B_2547_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_B_mode[] = {
	{
		sensor_s5kgn3_setfile_B_channel,	ARRAY_SIZE(sensor_s5kgn3_setfile_B_channel),
		sensor_s5kgn3_setfile_B_mipi_setting,	ARRAY_SIZE(sensor_s5kgn3_setfile_B_mipi_setting)
	},
};

/*************************************/
/* ========== Mode C =============== */
/*************************************/
enum {
	CAM_S5KGN3_SET_C_3315_MHZ = 0,
	CAM_S5KGN3_SET_C_3456_MHZ = 1,
	CAM_S5KGN3_SET_C_3340_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_C_3315_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x0103, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_C_3456_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x010E, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_C_3340_MHZ_REG_ARRAY[] = {
	  {0x0310, 0x0105, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_C_3315_mhz[] = {
    { MIPI_C_3315_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_3315_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_C_3456_mhz[] = {
    { MIPI_C_3456_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_3456_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5kgn3_setfile_C_3340_mhz[] = {
    { MIPI_C_3340_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_3340_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5kgn3_setfile_C_mipi_setting[] = {
	{ "3315 Mhz",
	  sensor_s5kgn3_setfile_C_3315_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_C_3315_mhz) },
	{ "3456 Mhz",
	  sensor_s5kgn3_setfile_C_3456_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_C_3456_mhz) },
	{ "3340 Mhz",
	  sensor_s5kgn3_setfile_C_3340_mhz, ARRAY_SIZE(sensor_s5kgn3_setfile_C_3340_mhz) },
};

static const struct cam_mipi_channel sensor_s5kgn3_setfile_C_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10618, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10619, 10652, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10653, 10693, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10694, 10776, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10777, 10816, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10817, 10838, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9671, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9672, 9746, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9747, 9829, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9830, 9904, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9905, 9938, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1221, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1222, 1237, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1238, 1322, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1323, 1385, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1386, 1396, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1397, 1480, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1481, 1513, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1593, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1594, 1627, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1628, 1668, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1669, 1738, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4458, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2269, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2270, 2335, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2336, 2344, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2345, 2427, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2428, 2500, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2501, 2563, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2953, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2954, 2976, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2977, 3077, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3078, 3088, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 763, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 136, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 137, 204, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 205, 286, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 287, 452, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 453, 533, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 534, 599, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 642, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 643, 792, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 793, 958, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 959, 1108, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1109, 1199, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1229, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1230, 1342, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1343, 1374, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1375, 1545, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1546, 1671, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1672, 1692, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1693, 1861, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1862, 1949, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2086, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2087, 2154, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2155, 2236, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2237, 2399, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2633, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2634, 2649, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2838, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2839, 2970, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2971, 2988, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2989, 3154, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3155, 3300, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3301, 3304, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3305, 3449, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3505, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3506, 3550, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3551, 3754, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3755, 3799, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5064, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5065, 5179, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5214, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5215, 5279, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5360, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5361, 5379, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5734, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5735, 5849, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5857, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5858, 5897, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5898, 5999, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6149, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6216, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6217, 6228, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6229, 6322, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6323, 6449, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6535, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6536, 6563, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6564, 6599, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8082, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8083, 8232, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8233, 8398, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8399, 8548, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8549, 8689, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8707, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8708, 8747, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8748, 8816, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8817, 9023, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9024, 9039, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9290, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9291, 9395, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9396, 9440, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9441, 9606, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9607, 9618, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9619, 9659, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9668, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9669, 9769, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9867, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9868, 9869, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10006, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10007, 10120, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10121, 10156, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10157, 10322, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10323, 10359, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36224, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36225, 36349, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37812, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37813, 37857, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37858, 38022, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38023, 38141, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38142, 38172, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38173, 38249, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38300, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38301, 38309, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38310, 38476, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38477, 38626, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38627, 38649, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38787, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38788, 38929, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38930, 38931, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38932, 39097, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39098, 39109, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39110, 39247, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39248, 39412, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39413, 39437, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39438, 39562, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39563, 39649, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39715, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39716, 39794, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39795, 39865, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39866, 40031, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40032, 40123, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40124, 40181, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40182, 40347, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40348, 40452, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40453, 40497, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40498, 40662, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40663, 40781, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40782, 40812, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40813, 40978, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40979, 41110, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41111, 41128, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41129, 41294, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41295, 41440, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41441, 41444, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41445, 41589, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41606, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41607, 41772, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41773, 41910, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41911, 41922, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41923, 42137, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42138, 42225, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42226, 42237, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42238, 42403, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42404, 42553, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42554, 42719, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42720, 42869, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42870, 43041, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43042, 43185, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43186, 43364, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43365, 43500, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43501, 43589, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55335, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55336, 55514, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55515, 55650, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55651, 55836, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55837, 55966, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55967, 56159, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56160, 56282, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56283, 56481, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56482, 56597, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56598, 56739, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66572, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66573, 66640, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66641, 66722, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66723, 66888, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66889, 66969, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66970, 67038, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67039, 67205, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67206, 67298, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67299, 67335, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68647, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68648, 68813, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68814, 68886, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68887, 68935, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 178380, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178381, 178780, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 185960, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185961, 186990, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186991, 191000, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191001, 191980, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 146800, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 146801, 149200, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 159440, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159441, 159620, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159621, 161630, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 161631, 164180, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 153120, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153121, 155290, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155291, 159440, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159441, 159620, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159621, 160580, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 124700, CAM_S5KGN3_SET_C_3456_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124701, 127860, CAM_S5KGN3_SET_C_3315_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 127861, 129420, CAM_S5KGN3_SET_C_3340_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129421, 130380, CAM_S5KGN3_SET_C_3456_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_C_mode[] = {
	{
		sensor_s5kgn3_setfile_C_channel,	ARRAY_SIZE(sensor_s5kgn3_setfile_C_channel),
		sensor_s5kgn3_setfile_C_mipi_setting,	ARRAY_SIZE(sensor_s5kgn3_setfile_C_mipi_setting)
	},
};


/*************************************/
/* ========== DUMMY ================ */
/*************************************/
enum {
	CAM_WIDE_SET_DUMMY_MHZ = 0,
};

struct cam_sensor_i2c_reg_array MIPI_WIDE_DUMMY_MHZ_REG_ARRAY[] = {
};

static const struct cam_sensor_i2c_reg_setting sensor_wide_setfile_dummy_mhz[] = {
    { MIPI_WIDE_DUMMY_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_WIDE_DUMMY_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_wide_setfile_dummy_mipi_setting[] = {
	{ "DUMMY Mhz",
	  sensor_wide_setfile_dummy_mhz, ARRAY_SIZE(sensor_wide_setfile_dummy_mhz) },
};

static const struct cam_mipi_channel sensor_wide_setfile_dummy_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_WIDE_SET_DUMMY_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_D_mode[] = {
	{
		sensor_wide_setfile_dummy_channel,	ARRAY_SIZE(sensor_wide_setfile_dummy_channel),
		sensor_wide_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_wide_setfile_dummy_mipi_setting)
	},
};
#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_S5KGN3_H_ */
