/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_S5KHP2_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_S5KHP2_H_

#include "cam_sensor_dev.h"

int num_wide_mipi_setting = 4;

/*
SET A : A01~14, B01, B06 (3712 / 3686.4)
SET B : C01 (3712)
SET C : B04, D02 (2860.8 / 2944 / 2988.8)
SET D : D01, D03 (1811.2 / 1849.6 / 1862.4)
*/

/*************************************/
/* ========== Mode A =============== */
/*************************************/

enum {
	CAM_S5KHP2_SET_A_3712_MHZ = 0,
	CAM_S5KHP2_SET_A_3686_MHZ = 1,
};

struct cam_sensor_i2c_reg_array MIPI_A_3712_MHZ_REG_ARRAY[] = {
    {0x0310, 0x0122, 0x00, 0x00},
	{0x0312, 0x0000, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_A_3686_MHZ_REG_ARRAY[] = {
    {0x0310, 0x0120, 0x00, 0x00},
	{0x0312, 0x0000, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_A_3712_mhz[] = {
    { MIPI_A_3712_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_A_3712_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_A_3686_mhz[] = {
    { MIPI_A_3686_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_A_3686_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5khp2_setfile_A_mipi_setting[] = {
	{ "3712 Msps",
	  sensor_s5khp2_setfile_A_3712_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_A_3712_mhz) },
	{ "3686 Msps",
	  sensor_s5khp2_setfile_A_3686_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_A_3686_mhz) },
};

static const struct cam_mipi_channel sensor_s5khp2_setfile_A_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10570, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10571, 10640, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10641, 10791, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10792, 10838, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9756, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9757, 9938, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1305, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1306, 1372, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1373, 1439, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1440, 1513, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1545, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1546, 1615, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1616, 1738, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4404, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4405, 4458, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 4387, 2281, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2282, 2347, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2348, 2417, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2418, 2563, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 3088, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 763, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 41, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 42, 181, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 182, 483, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 484, 599, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 813, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 814, 1199, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1511, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1512, 1645, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1646, 1779, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1780, 1949, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 1991, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1992, 2131, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2132, 2399, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2417, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2418, 2518, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2519, 2649, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2773, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2774, 2862, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2863, 2994, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2995, 3134, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3135, 3449, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3784, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3785, 3799, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5179, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5309, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5310, 5379, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5957, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5958, 5999, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6058, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6059, 6149, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6449, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6599, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8253, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8254, 8689, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8807, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8808, 8908, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8909, 9039, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9239, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9240, 9336, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9337, 9459, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9460, 9557, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9558, 9659, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9769, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9869, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9921, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9922, 10042, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10043, 10251, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10252, 10359, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36247, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36248, 36349, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37750, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37751, 38249, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38649, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38779, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38780, 38919, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38920, 39485, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39486, 39649, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 40176, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40177, 40390, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40391, 40913, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40914, 41002, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41003, 41134, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41135, 41274, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41275, 41589, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41686, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41687, 41988, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41989, 42104, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42105, 42320, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42321, 42460, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42461, 42930, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42931, 43012, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43013, 43093, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43094, 43233, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43234, 43589, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55243, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55244, 55383, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55384, 55906, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55907, 56156, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56157, 56732, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56733, 56739, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66477, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66478, 66617, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66618, 66919, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66920, 67059, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67060, 67335, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68935, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 176150, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176151, 178780, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 190680, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190681, 191980, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 164180, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 156320, CAM_S5KHP2_SET_A_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156321, 160580, CAM_S5KHP2_SET_A_3686_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 130380, CAM_S5KHP2_SET_A_3686_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_A_mode[] = {
	{
		sensor_s5khp2_setfile_A_channel,	ARRAY_SIZE(sensor_s5khp2_setfile_A_channel),
		sensor_s5khp2_setfile_A_mipi_setting,	ARRAY_SIZE(sensor_s5khp2_setfile_A_mipi_setting)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
enum {
	CAM_S5KHP2_SET_B_3712_MHZ = 0,
};

struct cam_sensor_i2c_reg_array MIPI_B_3712_MHZ_REG_ARRAY[] = {
    {0x0310, 0x0122, 0x00, 0x00},
	{0x0312, 0x0000, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};


static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_B_3712_mhz[] = {
    { MIPI_B_3712_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_B_3712_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};


static const struct cam_mipi_setting sensor_s5khp2_setfile_B_mipi_setting[] = {
	{ "3712 Msps",
	  sensor_s5khp2_setfile_B_3712_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_B_3712_mhz) },
};

static const struct cam_mipi_channel sensor_s5khp2_setfile_B_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10594, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10595, 10716, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10717, 10786, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10787, 10838, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9764, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9765, 9890, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9891, 9938, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1175, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1176, 1497, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1498, 1513, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1569, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1570, 1691, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1692, 1738, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4449, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4450, 4458, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2417, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2418, 2487, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2488, 2529, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2530, 2563, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2971, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2972, 3038, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3039, 3088, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 763, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 89, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 90, 333, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 334, 473, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 474, 599, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 829, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 830, 1081, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1082, 1199, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1251, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1252, 1895, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1896, 1949, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2039, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2040, 2283, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2284, 2399, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2403, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2404, 2609, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2610, 2649, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 3135, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3136, 3275, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3276, 3359, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3360, 3449, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3543, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3544, 3677, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3678, 3799, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5179, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5279, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5943, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5944, 5999, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6149, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6249, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6250, 6373, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6374, 6449, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6599, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8269, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8270, 8521, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8522, 8689, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8793, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8794, 8999, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9000, 9039, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9416, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9417, 9527, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9528, 9639, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9640, 9659, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9727, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9728, 9769, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9859, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9860, 9869, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10225, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10226, 10359, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36349, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37851, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37852, 38103, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38104, 38249, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38335, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38336, 38649, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38897, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38898, 39015, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39016, 39099, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39100, 39239, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39240, 39649, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39707, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39708, 39847, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39848, 40491, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40492, 40743, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40744, 41275, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41276, 41415, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41416, 41499, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41500, 41589, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41708, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41709, 42072, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42073, 42156, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42157, 42464, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42465, 42604, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42605, 42800, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42801, 42906, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42907, 43360, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43361, 43500, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43501, 43584, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43585, 43589, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55510, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55511, 55650, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55651, 55734, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55735, 55874, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55875, 56406, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56407, 56658, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56659, 56739, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66525, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66526, 66769, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66770, 66909, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66910, 67217, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67218, 67335, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68645, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68646, 68757, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68758, 68935, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174610, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174611, 178780, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 186860, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186861, 190540, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 190541, 191980, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 160180, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 160181, 163660, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163661, 164180, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 155730, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155731, 157940, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157941, 160580, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 124580, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126121, 127820, CAM_S5KHP2_SET_B_3712_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 127821, 130380, CAM_S5KHP2_SET_B_3712_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_B_mode[] = {
	{
		sensor_s5khp2_setfile_B_channel,	ARRAY_SIZE(sensor_s5khp2_setfile_B_channel),
		sensor_s5khp2_setfile_B_mipi_setting,	ARRAY_SIZE(sensor_s5khp2_setfile_B_mipi_setting)
	},
};

/*************************************/
/* ========== Mode C =============== */
/*************************************/
enum {
	CAM_S5KHP2_SET_C_2860_MHZ = 0,
	CAM_S5KHP2_SET_C_2944_MHZ = 1,
	CAM_S5KHP2_SET_C_2988_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_C_2860_MHZ_REG_ARRAY[] = {
    {0x0310, 0x01BF, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_C_2944_MHZ_REG_ARRAY[] = {
    {0x0310, 0x01CC, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_C_2988_MHZ_REG_ARRAY[] = {
    {0x0310, 0x01D3, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_C_2860_mhz[] = {
    { MIPI_C_2860_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_2860_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_C_2944_mhz[] = {
    { MIPI_C_2944_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_2944_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_C_2988_mhz[] = {
    { MIPI_C_2988_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_C_2988_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5khp2_setfile_C_mipi_setting[] = {
	{ "2860 Msps",
	  sensor_s5khp2_setfile_C_2860_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_C_2860_mhz) },
	{ "2944 Msps",
	  sensor_s5khp2_setfile_C_2944_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_C_2944_mhz) },
	{ "2988 Msps",
	  sensor_s5khp2_setfile_C_2988_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_C_2988_mhz) },
};

static const struct cam_mipi_channel sensor_s5khp2_setfile_C_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10620, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10621, 10639, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10640, 10709, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10710, 10838, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9927, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9928, 9938, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1348, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1349, 1359, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1360, 1411, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1412, 1429, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1430, 1499, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1500, 1513, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1595, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1596, 1614, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1615, 1684, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1685, 1738, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4393, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4394, 4458, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4393, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4394, 4413, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2254, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2255, 2324, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2325, 2563, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2960, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2961, 3088, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 718, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 719, 763, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 1, 140, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 141, 178, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 179, 318, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 319, 599, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 1155, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1156, 1199, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1596, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1597, 1619, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1620, 1722, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1723, 1759, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1760, 1899, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1900, 1949, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 1950, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1951, 2090, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2091, 2128, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2129, 2268, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2269, 2399, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2497, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2498, 2649, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2771, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2772, 2809, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2810, 2949, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2950, 3449, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3521, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3522, 3799, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5121, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5122, 5146, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5147, 5179, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5261, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5262, 5279, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5379, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5791, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5792, 5816, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5817, 5849, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5859, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5860, 5897, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5898, 5906, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5907, 5999, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6037, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6038, 6149, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6449, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6504, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6505, 6599, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8595, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8596, 8689, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8709, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8710, 8747, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8748, 8756, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8757, 8887, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8888, 9039, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9387, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9388, 9410, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9411, 9527, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9528, 9659, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9676, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9677, 9769, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9823, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9824, 9861, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9862, 9869, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 10273, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10274, 10359, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36349, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 38131, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38132, 38249, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38593, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38594, 38649, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 39063, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39064, 39203, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39204, 39241, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39242, 39381, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39382, 39649, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39666, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39667, 40238, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40239, 40378, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40379, 40771, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40772, 40911, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40912, 40949, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40950, 41089, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41090, 41589, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41717, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41718, 42088, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42089, 42228, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42229, 42258, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42259, 42398, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42399, 42939, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42940, 43079, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43080, 43280, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43281, 43408, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43409, 43420, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43421, 43589, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55430, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55431, 55558, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55559, 55570, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55571, 55770, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55771, 55910, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55911, 56451, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56452, 56470, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56471, 56591, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56592, 56622, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56623, 56739, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66436, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66437, 66576, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66577, 66614, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66615, 66754, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66755, 67326, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67327, 67335, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68619, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68620, 68654, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68655, 68712, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68713, 68750, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68751, 68794, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68795, 68890, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68891, 68935, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174120, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174121, 176740, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176741, 178780, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187420, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187421, 191980, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 147020, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147021, 149200, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158940, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158941, 164180, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 151810, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151811, 154140, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154141, 156600, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156601, 158940, CAM_S5KHP2_SET_C_2944_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158941, 160580, CAM_S5KHP2_SET_C_2988_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 128560, CAM_S5KHP2_SET_C_2860_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 128561, 130380, CAM_S5KHP2_SET_C_2944_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_C_mode[] = {
	{
		sensor_s5khp2_setfile_C_channel,	ARRAY_SIZE(sensor_s5khp2_setfile_C_channel),
		sensor_s5khp2_setfile_C_mipi_setting,	ARRAY_SIZE(sensor_s5khp2_setfile_C_mipi_setting)
	},
};


/*************************************/
/* ========== Mode D =============== */
/*************************************/
enum {
	CAM_S5KHP2_SET_D_1811_MHZ = 0,
	CAM_S5KHP2_SET_D_1849_MHZ = 1,
	CAM_S5KHP2_SET_D_1862_MHZ = 2,
};

struct cam_sensor_i2c_reg_array MIPI_D_1811_MHZ_REG_ARRAY[] = {
    {0x0310, 0x011B, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_D_1849_MHZ_REG_ARRAY[] = {
    {0x0310, 0x0121, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_D_1862_MHZ_REG_ARRAY[] = {
    {0x0310, 0x0123, 0x00, 0x00},
	{0x0312, 0x0001, 0x00, 0x00},
	{0x030E, 0x0003, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_D_1811_mhz[] = {
    { MIPI_D_1811_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_D_1811_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_D_1849_mhz[] = {
    { MIPI_D_1849_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_D_1849_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_s5khp2_setfile_D_1862_mhz[] = {
    { MIPI_D_1862_MHZ_REG_ARRAY, ARRAY_SIZE(MIPI_D_1862_MHZ_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_s5khp2_setfile_D_mipi_setting[] = {
	{ "1811 Msps",
	  sensor_s5khp2_setfile_D_1811_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_D_1811_mhz) },
	{ "1849 Msps",
	  sensor_s5khp2_setfile_D_1849_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_D_1849_mhz) },
	{ "1862 Msps",
	  sensor_s5khp2_setfile_D_1862_mhz, ARRAY_SIZE(sensor_s5khp2_setfile_D_1862_mhz) },
};

static const struct cam_mipi_channel sensor_s5khp2_setfile_D_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10600, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10601, 10745, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10746, 10754, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10755, 10815, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10816, 10838, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9695, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9696, 9737, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9738, 9775, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9776, 9845, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9846, 9883, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9884, 9938, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1184, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1185, 1250, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1251, 1308, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1309, 1361, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1362, 1404, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1405, 1431, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1432, 1513, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1575, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1576, 1720, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1721, 1729, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1730, 1738, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4399, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4400, 4438, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4439, 4458, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4399, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4400, 4413, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2242, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2243, 2261, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2262, 2326, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2327, 2366, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2367, 2458, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2459, 2521, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2522, 2528, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2529, 2563, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2958, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2959, 2990, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2991, 3088, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 724, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 725, 763, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 100, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 101, 391, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 392, 408, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 409, 531, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 532, 599, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 691, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 692, 775, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 776, 851, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 852, 991, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 992, 1066, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1067, 1186, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1187, 1199, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1268, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1269, 1400, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1401, 1517, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1518, 1623, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1624, 1709, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1710, 1763, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1764, 1946, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1947, 1949, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2050, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2051, 2341, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2342, 2358, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2359, 2399, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2508, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2509, 2587, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2588, 2648, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2649, 2649, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2785, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2786, 2822, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2823, 2953, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2954, 3033, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3034, 3216, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3217, 3343, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3344, 3356, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3357, 3449, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3517, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3518, 3581, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3582, 3799, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5179, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5188, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5189, 5236, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5237, 5279, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5308, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5309, 5379, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5849, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5855, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5856, 5999, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6048, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6049, 6127, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6128, 6149, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6151, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6152, 6236, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6237, 6291, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6292, 6449, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6497, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6498, 6524, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6525, 6599, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8131, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8132, 8215, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8216, 8291, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8292, 8431, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8432, 8506, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8507, 8626, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8627, 8646, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8647, 8689, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8705, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8706, 8898, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8899, 8977, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8978, 9038, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9039, 9039, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9238, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9239, 9319, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9320, 9364, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9365, 9459, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9460, 9541, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9542, 9626, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9627, 9659, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9769, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9869, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9963, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9964, 10052, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10053, 10213, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10214, 10359, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36322, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36323, 36349, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37778, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37779, 37854, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37855, 37962, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37963, 37994, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37995, 38249, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38316, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38317, 38452, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38453, 38494, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38495, 38649, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38866, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38867, 38960, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38961, 39060, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39061, 39114, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39115, 39298, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39299, 39360, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39361, 39438, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39439, 39649, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39721, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39722, 39771, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39772, 40063, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40064, 40203, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40204, 40290, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40291, 40418, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40419, 40494, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40495, 40602, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40603, 40634, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40635, 40925, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40926, 40962, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40963, 41093, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41094, 41173, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41174, 41356, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41357, 41483, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41484, 41496, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41497, 41589, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41664, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41665, 41804, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41805, 42107, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42108, 42200, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42201, 42241, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42242, 42440, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42441, 42450, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42451, 42690, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42691, 43052, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43053, 43134, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43135, 43216, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43217, 43321, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43322, 43356, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43357, 43437, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43438, 43577, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43578, 43589, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55284, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55285, 55366, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55367, 55471, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55472, 55506, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55507, 55587, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55588, 55727, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55728, 56031, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56032, 56221, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56222, 56282, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56283, 56474, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56475, 56648, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56649, 56739, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66536, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66537, 66827, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66828, 66844, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66845, 66967, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66968, 67151, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67152, 67285, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67286, 67335, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68651, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68652, 68693, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68694, 68935, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174960, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174961, 178540, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178541, 178780, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 187340, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187341, 188620, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188621, 191980, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 149200, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 160920, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 160921, 162020, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162021, 164180, CAM_S5KHP2_SET_D_1862_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 160580, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 125700, CAM_S5KHP2_SET_D_1811_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125701, 126540, CAM_S5KHP2_SET_D_1849_MHZ },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126541, 130380, CAM_S5KHP2_SET_D_1862_MHZ },
};

static const struct cam_mipi_sensor_mode sensor_wide_mipi_D_mode[] = {
	{
		sensor_s5khp2_setfile_D_channel,	ARRAY_SIZE(sensor_s5khp2_setfile_D_channel),
		sensor_s5khp2_setfile_D_mipi_setting,	ARRAY_SIZE(sensor_s5khp2_setfile_D_mipi_setting)
	},
};

#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_S5KHP2_H_ */
