/* Copyright (c) 2017, The Linux Foundation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_
#define _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_

#include "cam_sensor_dev.h"

int num_tele_mipi_setting = 2;

/*
TeleCAM3x_Adaptive MIPI 2.0_IMX754_231026.xlsx
SET A : Full mode, 2-Binning 120fps (1660.8 / 1670.4 / 1708.8)
SET B : 2-Binning 240fps (1900.8 / 1910.4 / 1968)
*/

/*************************************/
/* ========== SET A =============== */
/*************************************/
struct cam_sensor_i2c_reg_array MIPI_A_1660_REG_ARRAY[] = {
    {0x030F, 0x07, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_A_1670_REG_ARRAY[] = {
    {0x030F, 0x0A, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_A_1708_REG_ARRAY[] = {
    {0x030F, 0x16, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_1660[] = {
    { MIPI_A_1660_REG_ARRAY, ARRAY_SIZE(MIPI_A_1660_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_1670[] = {
    { MIPI_A_1670_REG_ARRAY, ARRAY_SIZE(MIPI_A_1670_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_A_mipi_1708[] = {
    { MIPI_A_1708_REG_ARRAY, ARRAY_SIZE(MIPI_A_1708_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx754_setfile_A_mipi_setting[] = {
	{ "1660 Msps", 1660,
	  sensor_imx754_setfile_A_mipi_1660, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_1660) },
	{ "1670 Msps", 1670,
	  sensor_imx754_setfile_A_mipi_1670, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_1670) },
	{ "1708 Msps", 1708,
	  sensor_imx754_setfile_A_mipi_1708, ARRAY_SIZE(sensor_imx754_setfile_A_mipi_1708) },
};

static const struct cam_mipi_cell_ratings sensor_imx754_setfile_A_mipi_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, {14, 14, 7} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, {6, 6, 6} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, {3, 6, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10583, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10584, 10592, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10593, 10641, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10642, 10653, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10654, 10703, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10704, 10706, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10707, 10711, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10712, 10760, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10761, 10773, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10774, 10776, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10777, 10822, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10823, 10828, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10829, 10830, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10831, 10838, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9677, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9678, 9692, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9693, 9699, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9700, 9729, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9730, 9748, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9749, 9762, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9763, 9799, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9800, 9811, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9812, 9818, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9819, 9851, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9852, 9868, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9869, 9881, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9882, 9921, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9922, 9929, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9930, 9938, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1175, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1176, 1192, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1193, 1224, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1225, 1227, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1228, 1244, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1245, 1277, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1278, 1294, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1295, 1314, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1315, 1343, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1344, 1347, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1348, 1366, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1367, 1396, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1397, 1413, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1414, 1436, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1437, 1461, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1462, 1466, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1467, 1488, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1489, 1513, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1558, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1559, 1567, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1568, 1616, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1617, 1628, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1629, 1678, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1679, 1681, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1682, 1686, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1687, 1735, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1736, 1738, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4359, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4360, 4379, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4380, 4424, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4425, 4429, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4430, 4449, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4450, 4458, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4413, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2257, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2258, 2272, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2273, 2284, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2285, 2327, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2328, 2333, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2334, 2342, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2343, 2376, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2377, 2394, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2395, 2403, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2404, 2446, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2447, 2453, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2454, 2464, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2465, 2495, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2496, 2516, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2517, 2523, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2524, 2563, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2961, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2962, 2973, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2974, 2988, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2989, 3010, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3011, 3025, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3026, 3037, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3038, 3080, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3081, 3088, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 749, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 750, 754, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 755, 763, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 67, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 68, 85, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 86, 183, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 184, 207, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 208, 306, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 307, 312, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 313, 323, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 324, 420, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 421, 446, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 447, 452, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 453, 545, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 546, 556, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 557, 560, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 561, 599, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 655, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 656, 685, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 686, 698, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 699, 759, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 760, 797, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 798, 825, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 826, 899, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 900, 922, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 923, 937, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 938, 1003, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1004, 1036, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1037, 1062, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1063, 1143, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1144, 1159, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1160, 1176, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1177, 1199, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1215, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1216, 1251, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1252, 1284, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1285, 1348, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1349, 1355, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1356, 1388, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1389, 1454, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1455, 1488, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1489, 1528, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1529, 1586, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1587, 1594, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1595, 1632, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1633, 1693, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1694, 1726, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1727, 1772, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1773, 1823, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1824, 1833, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1834, 1876, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1877, 1931, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1932, 1949, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 2017, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2018, 2035, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2036, 2133, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2134, 2157, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2158, 2256, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2257, 2262, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2263, 2273, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2274, 2370, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2371, 2396, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2397, 2399, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2418, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2419, 2428, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2429, 2469, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2470, 2558, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2559, 2568, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2569, 2609, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2610, 2649, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2815, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2816, 2844, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2845, 2869, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2870, 2955, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2956, 2967, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2968, 2984, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2985, 3052, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3053, 3088, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3089, 3107, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3108, 3192, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3193, 3206, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3207, 3228, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3229, 3290, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3291, 3332, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3333, 3346, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3347, 3430, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3431, 3445, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3446, 3449, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3523, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3524, 3546, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3547, 3576, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3577, 3620, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3621, 3650, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3651, 3675, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3676, 3760, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3761, 3790, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3791, 3799, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5047, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5048, 5113, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5114, 5144, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5145, 5179, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5187, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5188, 5217, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5218, 5242, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5243, 5279, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5337, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5338, 5362, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5363, 5379, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5783, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5784, 5814, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5815, 5849, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5861, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5862, 5863, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5864, 5910, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5911, 5958, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5959, 5968, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5969, 5999, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6009, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6010, 6098, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6099, 6108, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6109, 6149, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6184, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6185, 6225, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6226, 6236, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6237, 6283, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6284, 6365, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6366, 6376, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6377, 6423, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6424, 6449, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6451, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6452, 6454, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6455, 6508, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6509, 6556, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6557, 6594, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6595, 6599, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8095, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8096, 8125, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8126, 8138, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8139, 8199, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8200, 8237, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8238, 8265, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8266, 8339, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8340, 8362, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8363, 8377, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8378, 8443, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8444, 8476, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8477, 8502, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8503, 8583, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8584, 8599, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8600, 8616, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8617, 8687, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8688, 8689, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8711, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8712, 8713, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8714, 8760, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8761, 8808, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8809, 8818, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8819, 8859, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8860, 8948, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8949, 8958, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8959, 8999, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9000, 9039, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9267, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9268, 9292, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9293, 9336, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9337, 9371, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9372, 9389, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9390, 9434, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9435, 9511, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9512, 9529, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9530, 9574, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9575, 9615, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9616, 9626, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9627, 9659, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9677, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9678, 9718, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9719, 9743, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9744, 9769, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9774, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9775, 9824, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9825, 9828, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9829, 9869, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9942, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9943, 9976, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9977, 10026, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10027, 10039, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10040, 10116, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10117, 10124, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10125, 10179, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10180, 10220, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10221, 10264, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10265, 10277, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10278, 10359, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36214, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36215, 36291, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36292, 36313, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36314, 36336, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36337, 36349, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37751, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37752, 37841, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37842, 37856, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37857, 37891, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37892, 37981, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37982, 37990, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37991, 37996, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37997, 38078, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38079, 38100, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38101, 38130, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38131, 38218, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38219, 38229, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38230, 38240, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38241, 38249, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38263, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38264, 38316, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38317, 38360, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38361, 38371, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38372, 38420, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38421, 38470, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38471, 38500, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38501, 38560, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38561, 38597, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38598, 38610, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38611, 38649, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38666, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38667, 38726, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38727, 38733, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38734, 38770, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38771, 38831, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38832, 38866, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38867, 38910, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38911, 38965, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38966, 38971, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38972, 39014, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39015, 39068, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39069, 39105, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39106, 39154, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39155, 39204, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39205, 39208, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39209, 39259, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39260, 39305, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39306, 39344, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39345, 39399, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39400, 39442, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39443, 39445, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39446, 39503, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39504, 39542, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39543, 39582, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39583, 39643, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39644, 39649, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39659, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39660, 39671, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39672, 39675, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39676, 39763, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39764, 39769, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39770, 39815, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39816, 39903, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39904, 39909, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39910, 39914, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39915, 40006, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40007, 40007, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40008, 40054, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40055, 40146, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40147, 40147, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40148, 40153, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40154, 40243, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40244, 40251, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40252, 40293, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40294, 40383, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40384, 40391, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40392, 40481, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40482, 40496, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40497, 40531, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40532, 40621, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40622, 40630, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40631, 40636, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40637, 40718, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40719, 40740, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40741, 40770, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40771, 40858, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40859, 40869, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40870, 40880, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40881, 40955, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40956, 40984, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40985, 41009, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41010, 41095, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41096, 41107, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41108, 41124, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41125, 41192, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41193, 41228, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41229, 41247, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41248, 41332, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41333, 41346, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41347, 41368, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41369, 41430, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41431, 41472, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41473, 41486, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41487, 41570, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41571, 41585, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41586, 41589, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41591, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41592, 41643, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41644, 41685, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41686, 41696, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41697, 41783, {2, 2, 40} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41784, 41825, {2, 0, 40} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41826, 41836, {0, 0, 40} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41837, 41882, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41883, 41922, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41923, 41940, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41941, 42022, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42023, 42062, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42063, 42080, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42081, 42121, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42122, 42159, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42160, 42184, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42185, 42261, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42262, 42299, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42300, 42324, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42325, 42359, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42360, 42396, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42397, 42428, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42429, 42499, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42500, 42536, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42537, 42568, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42569, 42598, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42599, 42634, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42635, 42672, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42673, 42738, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42739, 42774, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42775, 42812, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42813, 42837, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42838, 42871, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42872, 42916, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42917, 42977, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42978, 43011, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43012, 43056, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43057, 43075, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43076, 43108, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43109, 43160, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43161, 43215, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43216, 43248, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43249, 43300, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43301, 43314, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43315, 43345, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43346, 43404, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43405, 43454, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43455, 43485, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43486, 43544, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43545, 43552, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43553, 43583, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43584, 43589, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55258, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55259, 55310, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55311, 55365, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55366, 55398, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55399, 55450, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55451, 55464, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55465, 55495, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55496, 55554, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55555, 55604, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55605, 55635, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55636, 55694, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55695, 55702, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55703, 55733, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55734, 55798, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55799, 55842, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55843, 55873, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55874, 55938, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55939, 55941, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55942, 55970, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55971, 56043, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56044, 56081, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56082, 56110, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56111, 56180, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56181, 56183, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56184, 56207, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56208, 56287, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56288, 56320, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56321, 56347, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56348, 56418, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56419, 56427, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56428, 56444, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56445, 56531, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56532, 56558, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56559, 56584, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56585, 56657, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56658, 56671, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56672, 56682, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56683, 56739, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66503, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66504, 66521, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66522, 66619, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66620, 66643, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66644, 66742, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66743, 66748, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66749, 66759, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66760, 66856, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66857, 66882, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66883, 66888, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66889, 66981, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66982, 66992, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66993, 66996, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66997, 67093, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67094, 67121, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67122, 67132, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67133, 67219, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67220, 67233, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67234, 67236, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67237, 67330, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67331, 67335, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68588, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68589, 68654, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68655, 68690, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68691, 68692, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68693, 68751, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68752, 68788, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68789, 68832, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68833, 68891, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68892, 68928, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68929, 68935, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, {6, 3, 6} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, {6, 6, 9} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, {6, 6, 6} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, {3, 9, 9} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, {6, 0, 6} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, {9, 3, 6} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, {3, 6, 3} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, {14, 7, 14} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 422000, 423340, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 423341, 423700, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 423701, 425660, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 425661, 426140, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 426141, 428120, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 428121, 428240, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 428241, 428460, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 428461, 430400, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 430401, 430920, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 430921, 431040, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 431041, 432900, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 432901, 433120, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 433121, 433200, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 433201, 433980, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 386000, 387100, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 387101, 387700, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 387701, 387960, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 387961, 389180, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 389181, 389940, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 389941, 390500, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 390501, 391980, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 391981, 392440, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 392441, 392740, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 392741, 394060, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 394061, 394720, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 394721, 395240, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 395241, 396860, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 396861, 397180, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 397181, 397520, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 397521, 397980, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 361000, 361300, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 361301, 362020, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 362021, 362680, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 362681, 363960, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 363961, 364100, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 364101, 364760, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 364761, 366080, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 366081, 366760, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 366761, 367560, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 367561, 368720, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 368721, 368880, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 368881, 369640, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 369641, 370860, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 370861, 371520, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 371521, 372440, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 372441, 373460, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 373461, 373660, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 373661, 374520, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 374521, 375620, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 375621, 375980, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 174160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174161, 174360, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 174361, 175180, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 175181, 176960, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176961, 177160, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177161, 177980, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177981, 178780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 524000, 525300, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 525301, 525880, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 525881, 526380, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 526381, 528100, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 528101, 528340, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 528341, 528680, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 528681, 530040, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 530041, 530760, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 530761, 531140, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 531141, 532840, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 532841, 533120, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 533121, 533560, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 533561, 534800, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 534801, 535640, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 535641, 535920, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 535921, 537600, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 537601, 537900, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 537901, 538000, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 186460, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186461, 186920, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186921, 187520, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 187521, 188400, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188401, 189000, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189001, 189500, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189501, 191200, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191201, 191800, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191801, 191980, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 146540, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 146541, 147860, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147861, 148480, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 148481, 149200, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 149200, 149340, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 149341, 149940, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 149941, 150440, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 150441, 151200, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 151600, 152740, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 152741, 153240, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 153241, 153580, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172000, 172220, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172221, 172260, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172261, 173200, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 173201, 174160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 174161, 174360, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 174361, 175000, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158880, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158881, 159700, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159701, 159920, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159921, 160860, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 160861, 162500, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162501, 162720, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162721, 163660, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 163661, 164180, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 386000, 387100, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 387101, 387700, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 387701, 387960, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 387961, 389180, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 389181, 389940, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 389941, 390500, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 390501, 391980, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 391981, 392440, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 392441, 392740, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 392741, 394060, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 394061, 394720, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 394721, 395240, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 395241, 396860, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 396861, 397180, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 397181, 397520, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 397521, 398940, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 398941, 398980, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 171800, 172220, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 172221, 172260, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 172261, 173200, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 173201, 174160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 174161, 174360, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 174361, 175180, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 175181, 176960, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 176961, 177160, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 177161, 177980, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 177981, 178800, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 152740, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 152741, 153240, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153241, 154120, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154121, 154820, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154821, 155180, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 155181, 156080, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156081, 157620, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157621, 157980, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 157981, 158880, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158881, 159700, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159701, 159920, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159921, 160580, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 143400, 143740, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 143741, 144560, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 144561, 145060, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 145061, 145580, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 470000, 470080, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 470081, 471080, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 471081, 471160, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 471161, 471980, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 514000, 514020, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 514021, 515820, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 515821, 516120, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 516121, 516820, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 516821, 518620, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 518621, 518800, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 518801, 518920, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 518921, 520560, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 520561, 521000, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 521001, 521600, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 521601, 523360, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 523361, 523580, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 523581, 523800, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 523801, 523980, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 376000, 376260, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 376261, 377320, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 377321, 378200, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 378201, 378420, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 378421, 379400, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 379401, 380400, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 380401, 381000, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 381001, 382200, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 382201, 382940, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 382941, 383200, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 383201, 384000, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 460000, 460320, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 460321, 461520, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 461521, 461660, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 461661, 462400, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 462401, 463620, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 463621, 464320, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 464321, 465200, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 465201, 466300, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 466301, 466420, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 466421, 467280, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 467281, 468360, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 468361, 469100, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 469101, 470080, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 470081, 471080, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 471081, 471160, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 471161, 472180, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 472181, 473100, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 473101, 473880, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 473881, 474980, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 474981, 475840, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 475841, 475900, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 475901, 477060, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 477061, 477840, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 477841, 478640, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 478641, 479860, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 479861, 480000, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 499200, 499380, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 499381, 499620, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 499621, 499700, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 499701, 501460, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 501461, 501580, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 501581, 502500, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 502501, 504260, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 504261, 504380, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 504381, 504480, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 504481, 506320, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 506321, 506340, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 506341, 507280, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 507281, 509120, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 509121, 509140, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 509141, 509260, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 509261, 511060, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 511061, 511220, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 511221, 512060, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 512061, 513860, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 513861, 514020, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 514021, 515820, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 515821, 516120, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 516121, 516820, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 516821, 518620, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 518621, 518800, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 518801, 518920, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 518921, 520560, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 520561, 521000, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 521001, 521600, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 521601, 523360, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 523361, 523580, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 523581, 523800, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 523801, 525300, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 525301, 525880, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 525881, 526380, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 526381, 528100, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 528101, 528340, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 528341, 528680, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 528681, 530040, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 530041, 530760, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 530761, 531140, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 531141, 532840, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 532841, 533120, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 533121, 533560, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 533561, 534800, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 534801, 535640, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 535641, 535920, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 535921, 537600, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 537601, 537900, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 537901, 537980, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 636667, 636787, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 636788, 637127, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637128, 637487, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637488, 637707, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637708, 638067, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638068, 638147, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638148, 638367, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638368, 638747, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638748, 639087, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639088, 639287, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639288, 639687, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639688, 639747, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639748, 639947, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639948, 640387, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640388, 640667, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640668, 640887, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640888, 641307, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641308, 641327, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641328, 641527, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641528, 642007, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642008, 642267, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642268, 642467, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642468, 642927, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642928, 642947, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642948, 643107, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643108, 643647, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643648, 643867, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643868, 644047, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644048, 644507, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644508, 644567, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644568, 644687, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644688, 645267, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645268, 645447, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645448, 645627, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645628, 646107, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 646108, 646207, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 646208, 646267, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 646268, 646666, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 422000, 423340, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 423341, 423700, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 423701, 425660, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 425661, 426140, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 426141, 428120, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 428121, 428240, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 428241, 428460, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 428461, 430400, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 430401, 430920, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 430921, 431040, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 431041, 432900, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 432901, 433120, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 433121, 433200, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 433201, 435140, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 435141, 435700, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 435701, 435920, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 435921, 437660, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 437661, 437940, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 437941, 438000, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 438001, 439880, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 439881, 439980, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 123440, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123441, 124760, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124761, 125480, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125481, 125520, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125521, 126700, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126701, 127440, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 127441, 128320, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 128321, 129500, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129501, 130240, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 130241, 130380, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620000, 620000, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620001, 620160, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620161, 620320, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620321, 620660, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620661, 620860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620861, 620960, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620961, 621580, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621581, 621780, {20, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621781, 621900, {20, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621901, 622240, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622241, 622480, {0, 20, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622481, 622540, {0, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622541, 623180, {1, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623181, 623420, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623421, 623480, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623481, 623840, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623841, 624100, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624101, 624120, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624121, 624760, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624761, 625040, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625041, 625060, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625061, 625420, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625421, 625700, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625701, 625740, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625741, 626360, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626361, 626640, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626641, 626660, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626661, 627020, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627021, 627300, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627301, 627360, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627361, 627940, {1, 1, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627941, 628220, {1, 0, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628221, 628300, {0, 0, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628301, 628600, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628601, 628880, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628881, 629000, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629001, 629540, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629541, 629800, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629801, 629920, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629921, 630200, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630201, 630460, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630461, 630620, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630621, 631140, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631141, 631380, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631381, 631560, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631561, 631780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631781, 632040, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632041, 632240, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632241, 632720, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632721, 632960, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632961, 633180, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633181, 633380, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633381, 633620, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633621, 633880, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633881, 634320, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634321, 634560, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634561, 634800, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634801, 634980, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634981, 635200, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635201, 635500, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635501, 635900, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635901, 636140, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636141, 636440, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636441, 636560, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636561, 636780, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636781, 637120, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637121, 637500, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637501, 637720, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637721, 638060, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638061, 638160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638161, 638360, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638361, 638760, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638761, 639080, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639081, 639300, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639301, 639680, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639681, 639740, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639741, 639940, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639941, 640380, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640381, 640680, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640681, 640880, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640881, 641320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641321, 641340, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641341, 641520, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641521, 642020, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642021, 642260, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642261, 642460, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642461, 642920, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642921, 642940, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642941, 643100, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643101, 643640, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643641, 643860, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643861, 644040, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644041, 644520, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644521, 644580, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644581, 644680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644681, 645260, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645261, 645440, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645441, 645620, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645621, 646100, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646101, 646200, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646201, 646280, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646281, 646900, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646901, 647040, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647041, 647200, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647201, 647700, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647701, 647820, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647821, 647860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647861, 648520, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648521, 648640, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648641, 648780, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648781, 649280, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649281, 649440, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649441, 649460, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649461, 650140, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650141, 650220, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650221, 650360, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650361, 650880, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650881, 651020, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651021, 651080, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651081, 651780, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651781, 651820, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651821, 651940, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651941, 652460, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652461, 652600, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652601, 652700, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652701, 653400, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 653401, 653540, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 653541, 654060, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654061, 654180, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654181, 654340, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654341, 655000, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655001, 655020, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655021, 655120, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655121, 655660, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655661, 655760, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655761, 655960, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655961, 656580, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656581, 656660, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656661, 656700, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656701, 657240, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657241, 657340, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657341, 657600, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657601, 658180, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658181, 658280, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658281, 658840, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658841, 658920, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658921, 659220, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659221, 659760, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659761, 659860, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659861, 659920, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659921, 660420, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660421, 660500, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660501, 660840, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660841, 661360, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661361, 661440, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661441, 661540, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661541, 662020, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662021, 662080, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662081, 662480, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662481, 662940, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662941, 663020, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663021, 663160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663161, 663600, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663601, 663660, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663661, 664100, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664101, 664540, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664541, 664600, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664601, 664800, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664801, 665200, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665201, 665260, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665261, 665720, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665721, 666120, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666121, 666180, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666181, 666420, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666421, 666780, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666781, 666840, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666841, 667360, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667361, 667720, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667721, 667760, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667761, 668040, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668041, 668380, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668381, 668420, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668421, 668980, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668981, 669320, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669321, 669340, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669341, 669680, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669681, 669960, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669961, 670000, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670001, 670600, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670601, 670900, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670901, 670920, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670921, 671300, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671301, 671560, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671561, 671580, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671581, 672240, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672241, 672500, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672501, 672520, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672521, 672940, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672941, 673160, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 673161, 673860, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 673861, 674080, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674081, 674100, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674101, 674560, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674561, 674740, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674741, 675500, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675501, 675680, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675681, 676180, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676181, 676320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676321, 676340, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676341, 677120, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677121, 677260, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677261, 677820, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677821, 677900, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677901, 677920, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677921, 678740, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678741, 678840, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678841, 678860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678861, 679440, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 679441, 679480, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 679481, 679520, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 679521, 680000, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620000, 620000, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620001, 620160, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620161, 620320, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620321, 620660, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620661, 620860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620861, 620960, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620961, 621580, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621581, 621780, {20, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621781, 621900, {20, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621901, 622240, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622241, 622480, {0, 20, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622481, 622540, {0, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622541, 623180, {1, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623181, 623420, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623421, 623480, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623481, 623840, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623841, 624100, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624101, 624120, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624121, 624760, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624761, 625040, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625041, 625060, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625061, 625420, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625421, 625700, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625701, 625740, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625741, 626360, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626361, 626640, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626641, 626660, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626661, 627020, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627021, 627300, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627301, 627360, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627361, 627940, {1, 1, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627941, 628220, {1, 0, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628221, 628300, {0, 0, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628301, 628600, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628601, 628880, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628881, 629000, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629001, 629540, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629541, 629800, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629801, 629920, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629921, 630200, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630201, 630460, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630461, 630620, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630621, 631140, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631141, 631380, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631381, 631560, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631561, 631780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631781, 632040, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632041, 632240, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632241, 632720, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632721, 632960, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632961, 633180, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633181, 633380, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633381, 633620, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633621, 633880, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633881, 634320, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634321, 634560, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634561, 634800, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634801, 634980, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634981, 635200, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635201, 635500, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635501, 635900, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635901, 636140, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636141, 636440, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636441, 636560, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636561, 636780, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636781, 637120, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637121, 637500, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637501, 637720, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637721, 638060, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638061, 638160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638161, 638360, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638361, 638760, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638761, 639080, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639081, 639300, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639301, 639680, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639681, 639740, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639741, 639940, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639941, 640380, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640381, 640680, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640681, 640880, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640881, 641320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641321, 641340, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641341, 641520, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641521, 642020, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642021, 642260, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642261, 642460, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642461, 642920, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642921, 642940, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642941, 643100, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643101, 643640, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643641, 643860, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643861, 644040, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644041, 644520, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644521, 644580, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644581, 644680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644681, 645260, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645261, 645440, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645441, 645620, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645621, 646100, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646101, 646200, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646201, 646280, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646281, 646900, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646901, 647040, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647041, 647200, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647201, 647700, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647701, 647820, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647821, 647860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647861, 648520, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648521, 648640, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648641, 648780, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648781, 649280, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649281, 649440, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649441, 649460, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649461, 650140, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650141, 650220, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650221, 650360, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650361, 650880, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650881, 651020, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651021, 651080, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651081, 651780, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651781, 651820, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651821, 651940, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651941, 652460, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652461, 652600, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652601, 652700, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652701, 653333, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693334, 693394, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693395, 693714, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693715, 693834, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693835, 694094, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694095, 694654, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694655, 694774, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694775, 695014, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695015, 695314, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695315, 695414, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695415, 695714, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695715, 696234, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696235, 696354, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696355, 696654, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696655, 696894, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696895, 697014, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697015, 697334, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697335, 697814, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697815, 697954, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697955, 698274, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698275, 698474, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698475, 698614, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698615, 698974, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698975, 699394, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699395, 699534, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699535, 699914, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699915, 700054, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700055, 700194, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700195, 700594, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700595, 700974, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700975, 701134, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701135, 701534, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701535, 701634, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701635, 701794, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701795, 702234, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702235, 702574, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702575, 702714, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702715, 703154, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703155, 703214, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703215, 703374, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703375, 703854, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703855, 704154, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704155, 704314, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704315, 704794, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704795, 704974, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704975, 705474, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705475, 705734, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705735, 705894, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705895, 706374, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706375, 706414, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706415, 706554, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706555, 707114, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707115, 707314, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707315, 707494, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707495, 707954, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707955, 708034, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708035, 708154, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708155, 708734, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708735, 708894, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708895, 709074, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709075, 709534, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709535, 709674, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709675, 709734, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709735, 710354, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710355, 710474, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710475, 710674, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710675, 711114, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711115, 711294, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711295, 711334, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711335, 711994, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711995, 712054, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712055, 712274, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712275, 712694, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712695, 712914, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712915, 713614, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713615, 713634, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713635, 713854, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713855, 714294, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714295, 714514, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714515, 714554, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714555, 715214, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715215, 715254, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715255, 715454, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715455, 715874, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715875, 716114, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716115, 716174, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716175, 716794, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716795, 716874, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716875, 717034, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717035, 717454, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717455, 717694, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717695, 717814, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717815, 718374, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718375, 718494, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718495, 718634, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718635, 719034, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719035, 719294, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719295, 719434, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719435, 719954, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719955, 720134, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720135, 720214, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720215, 720614, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720615, 720874, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720875, 721054, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721055, 721554, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721555, 721754, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721755, 721814, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721815, 722194, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722195, 722474, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722475, 722694, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722695, 723134, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723135, 723374, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723375, 723394, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723395, 723774, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723775, 724054, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724055, 724314, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724315, 724714, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724715, 724994, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724995, 725014, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725015, 725354, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725355, 725654, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725655, 725934, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725935, 726294, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726295, 726574, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726575, 726634, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726635, 726934, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726935, 727234, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727235, 727574, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727575, 727874, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727875, 728174, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728175, 728274, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728275, 728514, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728515, 728834, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728835, 729194, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729195, 729454, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729455, 729774, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729775, 729894, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729895, 730094, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730095, 730414, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730415, 730834, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730835, 731034, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731035, 731354, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731355, 731514, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731515, 731694, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731695, 732014, {20, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732015, 732454, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732455, 732614, {20, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732615, 732954, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732955, 733154, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 733155, 733274, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 733275, 733333, {1, 0, 1} },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_A_mode[] = {
	{
		sensor_imx754_setfile_A_mipi_channel,	ARRAY_SIZE(sensor_imx754_setfile_A_mipi_channel),
		sensor_imx754_setfile_A_mipi_setting,	ARRAY_SIZE(sensor_imx754_setfile_A_mipi_setting)
	},
};

/*************************************/
/* ========== Mode B =============== */
/*************************************/
struct cam_sensor_i2c_reg_array MIPI_B_1900_REG_ARRAY[] = {
    {0x030F, 0x52, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_B_1910_REG_ARRAY[] = {
    {0x030F, 0x55, 0x00, 0x00},
};

struct cam_sensor_i2c_reg_array MIPI_B_1968_REG_ARRAY[] = {
    {0x030F, 0x67, 0x00, 0x00},
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_1900[] = {
    { MIPI_B_1900_REG_ARRAY, ARRAY_SIZE(MIPI_B_1900_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_1910[] = {
    { MIPI_B_1910_REG_ARRAY, ARRAY_SIZE(MIPI_B_1910_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_sensor_i2c_reg_setting sensor_imx754_setfile_B_mipi_1968[] = {
    { MIPI_B_1968_REG_ARRAY, ARRAY_SIZE(MIPI_B_1968_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_BYTE, 0 }
};

static const struct cam_mipi_setting sensor_imx754_setfile_B_mipi_setting[] = {
	{ "1900 Msps", 1900,
	  sensor_imx754_setfile_B_mipi_1900, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_1900) },
	{ "1910 Msps", 1910,
	  sensor_imx754_setfile_B_mipi_1910, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_1910) },
	{ "1968 Msps", 1968,
	  sensor_imx754_setfile_B_mipi_1968, ARRAY_SIZE(sensor_imx754_setfile_B_mipi_1968) },
};

static const struct cam_mipi_cell_ratings sensor_imx754_setfile_B_mipi_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, {14, 14, 14} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_002_GSM_EGSM900), 0, 0, {7, 14, 14} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_003_GSM_DCS1800), 0, 0, {6, 6, 6} },
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_004_GSM_PCS1900), 0, 0, {6, 6, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10562, 10577, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10578, 10608, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10609, 10625, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10626, 10648, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10649, 10678, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10679, 10690, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10691, 10718, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10719, 10745, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10746, 10760, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10761, 10788, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10789, 10815, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10816, 10826, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_011_WCDMA_WB01), 10827, 10838, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9662, 9664, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9665, 9674, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9675, 9723, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9724, 9734, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9735, 9740, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9741, 9789, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9790, 9804, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9805, 9810, {3, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9811, 9859, {0, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9860, 9874, {0, 0, 60} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9875, 9876, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9877, 9926, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_012_WCDMA_WB02), 9927, 9938, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1162, 1166, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1167, 1186, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1187, 1227, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1228, 1232, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1233, 1256, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1257, 1297, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1298, 1302, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1303, 1322, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1323, 1367, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1368, 1369, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1370, 1392, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1393, 1437, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1438, 1439, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1440, 1458, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1459, 1505, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1506, 1508, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_013_WCDMA_WB03), 1509, 1513, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1537, 1552, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1553, 1583, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1584, 1600, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1601, 1623, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1624, 1653, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1654, 1665, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1666, 1693, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1694, 1720, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1721, 1735, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_014_WCDMA_WB04), 1736, 1738, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4357, 4379, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4380, 4392, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4393, 4401, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4402, 4445, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_015_WCDMA_WB05), 4446, 4458, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4387, 4392, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4393, 4401, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_016_WCDMA_WB06), 4402, 4413, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2237, 2259, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2260, 2303, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2304, 2326, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2327, 2329, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2330, 2373, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2374, 2395, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2396, 2396, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2397, 2444, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2445, 2462, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2463, 2465, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2466, 2514, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2515, 2531, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2532, 2532, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_017_WCDMA_WB07), 2533, 2563, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2937, 2951, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2952, 2973, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2974, 2974, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 2975, 3016, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3017, 3040, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3041, 3044, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3045, 3086, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_018_WCDMA_WB08), 3087, 3088, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 712, 717, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 718, 726, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_2_WCDMA, CAM_BAND_029_WCDMA_WB19), 727, 763, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 0, 10, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 11, 55, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 56, 117, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 118, 150, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 151, 196, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 197, 257, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 258, 281, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 282, 336, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 337, 390, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 391, 421, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 422, 477, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 478, 530, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 531, 553, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_091_LTE_LB01), 554, 599, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 600, 606, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 607, 628, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 629, 649, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 650, 746, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 747, 768, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 769, 781, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 782, 879, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 880, 909, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 910, 921, {3, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 922, 1019, {0, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1020, 1049, {0, 0, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1050, 1052, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1053, 1152, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1153, 1191, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1192, 1192, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_092_LTE_LB02), 1193, 1199, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1200, 1213, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1214, 1232, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1233, 1273, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1274, 1354, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1355, 1365, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1366, 1413, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1414, 1494, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1495, 1505, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1506, 1544, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1545, 1635, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1636, 1638, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1639, 1684, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1685, 1775, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1776, 1778, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1779, 1816, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1817, 1911, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1912, 1916, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_093_LTE_LB03), 1917, 1949, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1950, 1960, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 1961, 2005, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2006, 2067, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2068, 2100, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2101, 2146, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2147, 2207, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2208, 2231, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2232, 2286, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2287, 2340, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2341, 2371, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_094_LTE_LB04), 2372, 2399, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2400, 2469, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2470, 2495, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2496, 2513, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2514, 2600, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2601, 2636, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2637, 2646, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_095_LTE_LB05), 2647, 2649, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2750, 2766, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2767, 2819, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2820, 2907, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2908, 2952, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2953, 2959, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 2960, 3047, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3048, 3091, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3092, 3092, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3093, 3188, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3189, 3225, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3226, 3231, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3232, 3328, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3329, 3362, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3363, 3365, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_097_LTE_LB07), 3366, 3449, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3450, 3502, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3503, 3547, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3548, 3549, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3550, 3633, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3634, 3681, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3682, 3688, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3689, 3773, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_098_LTE_LB08), 3774, 3799, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5010, 5018, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5019, 5099, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5100, 5121, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5122, 5158, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_102_LTE_LB12), 5159, 5179, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5180, 5240, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5241, 5253, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_103_LTE_LB13), 5254, 5279, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5280, 5360, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5361, 5373, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_104_LTE_LB14), 5374, 5379, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5730, 5769, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5770, 5791, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5792, 5828, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_107_LTE_LB17), 5829, 5849, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5850, 5869, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5870, 5895, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5896, 5913, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_108_LTE_LB18), 5914, 5999, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6000, 6009, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6010, 6035, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6036, 6053, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6054, 6140, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_109_LTE_LB19), 6141, 6149, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6150, 6181, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6182, 6184, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6185, 6224, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6225, 6316, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6317, 6323, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6324, 6357, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_110_LTE_LB20), 6358, 6449, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6450, 6461, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6462, 6495, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6496, 6571, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_111_LTE_LB21), 6572, 6599, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8040, 8046, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8047, 8068, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8069, 8089, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8090, 8186, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8187, 8208, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8209, 8221, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8222, 8319, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8320, 8349, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8350, 8361, {3, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8362, 8459, {0, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8460, 8489, {0, 0, 60} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8490, 8492, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8493, 8592, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8593, 8631, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8632, 8632, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_115_LTE_LB25), 8633, 8689, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8690, 8719, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8720, 8745, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8746, 8763, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8764, 8859, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8860, 8885, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8886, 8903, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8904, 8990, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 8991, 9026, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9027, 9036, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_116_LTE_LB26), 9037, 9039, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9210, 9290, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9291, 9303, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9304, 9341, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9342, 9431, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9432, 9434, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9435, 9474, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9475, 9571, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9572, 9574, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9575, 9614, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_118_LTE_LB28), 9615, 9659, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9660, 9729, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9730, 9751, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_119_LTE_LB29), 9752, 9769, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9770, 9810, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9811, 9815, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9816, 9824, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_120_LTE_LB30), 9825, 9869, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9920, 9934, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9935, 9949, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9950, 9993, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 9994, 10067, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10068, 10089, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10090, 10133, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10134, 10207, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10208, 10230, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10231, 10264, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10265, 10340, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_122_LTE_LB32), 10341, 10359, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36200, 36225, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36226, 36264, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36265, 36272, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_124_LTE_LB34), 36273, 36349, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37750, 37773, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37774, 37776, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37777, 37845, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37846, 37906, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37907, 37916, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37917, 37985, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 37986, 38046, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38047, 38048, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38049, 38126, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38127, 38179, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38180, 38188, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_128_LTE_LB38), 38189, 38249, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38250, 38256, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38257, 38351, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38352, 38356, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38357, 38387, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38388, 38483, {60, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38484, 38497, {60, 60, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38498, 38527, {60, 60, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38528, 38623, {0, 60, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38624, 38637, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_129_LTE_LB39), 38638, 38649, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38650, 38661, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38662, 38773, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38774, 38777, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38778, 38801, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38802, 38914, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38915, 38917, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38918, 38932, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 38933, 39050, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39051, 39054, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39055, 39072, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39073, 39190, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39191, 39195, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39196, 39204, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39205, 39323, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39324, 39335, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39336, 39344, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39345, 39463, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39464, 39475, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39476, 39477, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39478, 39596, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39597, 39615, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39616, 39617, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_130_LTE_LB40), 39618, 39649, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39650, 39728, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39729, 39741, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39742, 39781, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39782, 39868, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39869, 39873, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39874, 39922, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 39923, 40001, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40002, 40013, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40014, 40062, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40063, 40141, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40142, 40145, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40146, 40203, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40204, 40273, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40274, 40285, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40286, 40343, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40344, 40413, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40414, 40416, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40417, 40485, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40486, 40546, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40547, 40556, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40557, 40625, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40626, 40686, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40687, 40688, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40689, 40766, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40767, 40819, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40820, 40828, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40829, 40906, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40907, 40959, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 40960, 41047, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41048, 41092, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41093, 41099, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41100, 41187, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41188, 41231, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41232, 41232, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41233, 41328, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41329, 41365, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41366, 41371, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41372, 41468, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41469, 41502, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41503, 41505, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_131_LTE_LB41), 41506, 41589, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41590, 41602, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41603, 41634, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41635, 41678, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41679, 41734, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41735, 41774, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41775, 41819, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41820, 41874, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41875, 41907, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41908, 41959, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 41960, 42005, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42006, 42047, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42048, 42100, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42101, 42145, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42146, 42180, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42181, 42240, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42241, 42277, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42278, 42320, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42321, 42381, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42382, 42417, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42418, 42453, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42454, 42521, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42522, 42549, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42550, 42593, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42594, 42662, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42663, 42689, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42690, 42725, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42726, 42802, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42803, 42820, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42821, 42865, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42866, 42944, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42945, 42960, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42961, 42998, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 42999, 43084, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43085, 43092, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43093, 43138, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43139, 43225, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43226, 43232, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43233, 43271, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43272, 43363, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43364, 43365, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43366, 43411, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43412, 43503, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43504, 43506, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43507, 43544, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_132_LTE_LB42), 43545, 43589, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55240, 55242, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55243, 55288, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55289, 55375, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55376, 55382, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55383, 55421, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55422, 55513, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55514, 55515, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55516, 55561, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55562, 55653, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55654, 55656, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55657, 55694, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55695, 55785, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55786, 55796, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55797, 55834, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55835, 55925, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55926, 55937, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55938, 55967, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 55968, 56056, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56057, 56077, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56078, 56107, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56108, 56196, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56197, 56218, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56219, 56240, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56241, 56328, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56329, 56358, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56359, 56380, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56381, 56468, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56469, 56499, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56500, 56513, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56514, 56599, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56600, 56639, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56640, 56653, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_138_LTE_LB48), 56654, 56739, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66436, 66446, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66447, 66491, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66492, 66553, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66554, 66586, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66587, 66632, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66633, 66693, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66694, 66717, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66718, 66772, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66773, 66826, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66827, 66857, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66858, 66913, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66914, 66966, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66967, 66989, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 66990, 67053, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67054, 67099, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67100, 67129, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67130, 67195, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67196, 67239, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67240, 67260, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_156_LTE_LB66), 67261, 67335, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68586, 68591, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68592, 68623, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68624, 68671, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68672, 68731, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68732, 68763, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68764, 68812, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68813, 68863, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68864, 68895, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_3_LTE, CAM_BAND_161_LTE_LB71), 68896, 68935, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_051_TDSCDMA_TD1), 0, 0, {63, 63, 6} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_052_TDSCDMA_TD2), 0, 0, {6, 69, 9} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_053_TDSCDMA_TD3), 0, 0, {6, 63, 3} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_054_TDSCDMA_TD4), 0, 0, {3, 9, 3} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_055_TDSCDMA_TD5), 0, 0, {4, 4, 4} },
	{ CAM_RAT_BAND(CAM_RAT_4_TDSCDMA, CAM_BAND_056_TDSCDMA_TD6), 0, 0, {66, 3, 6} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_061_CDMA_BC0), 0, 0, {14, 21, 21} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_062_CDMA_BC1), 0, 0, {6, 6, 3} },
	{ CAM_RAT_BAND(CAM_RAT_5_CDMA, CAM_BAND_071_CDMA_BC10), 0, 0, {7, 14, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 422000, 422200, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 422201, 423100, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 423101, 424340, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 424341, 425000, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 425001, 425920, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 425921, 427140, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 427141, 427620, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 427621, 428720, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 428721, 429800, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 429801, 430420, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 430421, 431540, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 431541, 432600, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 432601, 433060, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_256_NR5G_N001), 433061, 433980, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 386000, 386120, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 386121, 386560, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 386561, 386980, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 386981, 388920, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 388921, 389360, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 389361, 389620, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 389621, 391580, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 391581, 392180, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 392181, 392420, {3, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 392421, 394380, {0, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 394381, 394980, {0, 0, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 394981, 395040, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 395041, 397040, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 397041, 397820, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 397821, 397840, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_257_NR5G_N002), 397841, 397980, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 361000, 361260, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 361261, 361640, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 361641, 362460, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 362461, 364080, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 364081, 364300, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 364301, 365260, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 365261, 366880, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 366881, 367100, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 367101, 367880, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 367881, 369700, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 369701, 369760, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 369761, 370680, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 370681, 372500, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 372501, 372560, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 372561, 373320, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 373321, 375220, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 375221, 375320, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_258_NR5G_N003), 375321, 375980, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 173800, 175180, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 175181, 175700, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 175701, 176060, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 176061, 177800, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 177801, 178520, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178521, 178720, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_260_NR5G_N005), 178721, 178780, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 524000, 524320, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 524321, 525380, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 525381, 527140, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 527141, 528040, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 528041, 528180, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 528181, 529940, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 529941, 530820, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 530821, 530840, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 530841, 532760, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 532761, 533500, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 533501, 533620, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 533621, 535560, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 535561, 536240, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 536241, 536300, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_262_NR5G_N007), 536301, 538000, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 185000, 186040, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186041, 186940, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186941, 186980, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 186981, 188660, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 188661, 189620, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189621, 189760, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 189761, 191460, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_263_NR5G_N008), 191461, 191980, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145800, 145960, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 145961, 147580, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 147581, 148020, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 148021, 148760, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_267_NR5G_N012), 148761, 149200, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 149200, 150400, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 150401, 150660, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_268_NR5G_N013), 150661, 151200, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 151600, 153200, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 153201, 153460, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_269_NR5G_N014), 153461, 153580, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172000, 172380, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172381, 172900, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 172901, 173260, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_273_NR5G_N018), 173261, 175000, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158200, 158820, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158821, 158880, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 158881, 159680, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 159681, 161520, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 161521, 161660, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 161661, 162340, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_275_NR5G_N020), 162341, 164180, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 386000, 386120, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 386121, 386560, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 386561, 386980, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 386981, 388920, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 388921, 389360, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 389361, 389620, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 389621, 391580, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 391581, 392180, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 392181, 392420, {3, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 392421, 394380, {0, 3, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 394381, 394980, {0, 0, 60} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 394981, 395040, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 395041, 397040, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 397041, 397820, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 397821, 397840, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_280_NR5G_N025), 397841, 398980, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 171800, 172380, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 172381, 172900, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 172901, 173260, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 173261, 175180, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 175181, 175700, {0, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 175701, 176060, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 176061, 177800, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 177801, 178520, {7, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 178521, 178720, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_281_NR5G_N026), 178721, 178800, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 151600, 153200, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153201, 153460, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 153461, 154220, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 154221, 156020, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156021, 156080, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156081, 156880, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 156881, 158820, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158821, 158880, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 158881, 159680, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_283_NR5G_N028), 159681, 160580, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 143400, 144780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 144781, 145220, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_284_NR5G_N029), 145221, 145580, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 470000, 470800, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 470801, 470900, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 470901, 471080, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_285_NR5G_N030), 471081, 471980, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 514000, 514460, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 514461, 514520, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 514521, 515900, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 515901, 517120, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 517121, 517320, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 517321, 518700, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 518701, 519920, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 519921, 519960, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 519961, 521520, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 521521, 522580, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 522581, 522760, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_293_NR5G_N038), 522761, 523980, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 376000, 376120, {3, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 376121, 378020, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 378021, 378120, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 378121, 378740, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 378741, 380660, {60, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 380661, 380940, {60, 60, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 380941, 381540, {60, 60, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 381541, 383460, {0, 60, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 383461, 383740, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_294_NR5G_N039), 383741, 384000, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 460000, 460220, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 460221, 462460, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 462461, 462540, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 462541, 463020, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 463021, 465280, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 465281, 465340, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 465341, 465640, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 465641, 468000, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 468001, 468080, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 468081, 468440, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 468441, 470800, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 470801, 470900, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 470901, 471080, {0, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 471081, 473460, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 473461, 473700, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 473701, 473880, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 473881, 476260, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 476261, 476500, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 476501, 476540, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 476541, 478920, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 478921, 479300, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 479301, 479340, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_295_NR5G_N040), 479341, 480000, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 499200, 500760, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 500761, 501020, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 501021, 501820, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 501821, 503560, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 503561, 503660, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 503661, 504640, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 504641, 506220, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 506221, 506460, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 506461, 507440, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 507441, 509020, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 509021, 509100, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 509101, 510260, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 510261, 511660, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 511661, 511900, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 511901, 513060, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 513061, 514460, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 514461, 514520, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 514521, 515900, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 515901, 517120, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 517121, 517320, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 517321, 518700, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 518701, 519920, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 519921, 519960, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 519961, 521520, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 521521, 522580, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 522581, 522760, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 522761, 524320, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 524321, 525380, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 525381, 527140, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 527141, 528040, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 528041, 528180, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 528181, 529940, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 529941, 530820, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 530821, 530840, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 530841, 532760, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 532761, 533500, {2, 0, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 533501, 533620, {2, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 533621, 535560, {0, 2, 2} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 535561, 536240, {0, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 536241, 536300, {2, 2, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_296_NR5G_N041), 536301, 537980, {2, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 636667, 636667, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 636668, 636987, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 636988, 637567, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637568, 637607, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637608, 637867, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 637868, 638487, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638488, 638807, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 638808, 639407, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639408, 639427, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639428, 639687, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 639688, 640287, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640288, 640367, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640368, 640627, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 640628, 641227, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641228, 641307, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641308, 641507, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 641508, 642107, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642108, 642247, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642248, 642447, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 642448, 643027, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643028, 643187, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643188, 643327, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643328, 643907, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 643908, 644107, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644108, 644267, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644268, 644847, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 644848, 645047, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645048, 645147, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645148, 645727, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645728, 645987, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 645988, 646087, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 646088, 646647, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_303_NR5G_N048), 646648, 646666, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 422000, 422200, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 422201, 423100, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 423101, 424340, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 424341, 425000, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 425001, 425920, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 425921, 427140, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 427141, 427620, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 427621, 428720, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 428721, 429800, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 429801, 430420, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 430421, 431540, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 431541, 432600, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 432601, 433060, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 433061, 434340, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 434341, 435260, {3, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 435261, 435860, {3, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 435861, 437180, {0, 3, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 437181, 438060, {0, 3, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 438061, 438480, {0, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_321_NR5G_N066), 438481, 439980, {3, 0, 3} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123400, 123500, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 123501, 124140, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 124141, 125100, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 125101, 126300, {7, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126301, 126940, {0, 7, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 126941, 127920, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 127921, 128940, {0, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 128941, 129580, {7, 0, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_326_NR5G_N071), 129581, 130380, {7, 7, 7} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620000, 620380, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620381, 620600, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620601, 620680, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 620681, 621320, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621321, 621500, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621501, 621620, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 621621, 622180, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622181, 622420, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622421, 622560, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 622561, 623120, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623121, 623320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623321, 623500, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 623501, 624000, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624001, 624240, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624241, 624440, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624441, 624940, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 624941, 625140, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625141, 625380, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625381, 625800, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 625801, 626060, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626061, 626320, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626321, 626740, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626741, 626960, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 626961, 627240, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627241, 627620, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627621, 627880, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 627881, 628180, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628181, 628560, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628561, 628780, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 628781, 629120, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629121, 629420, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629421, 629700, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 629701, 630060, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630061, 630360, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630361, 630600, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 630601, 631000, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631001, 631240, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631241, 631520, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631521, 631940, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 631941, 632180, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632181, 632420, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632421, 632860, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 632861, 633060, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633061, 633340, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633341, 633800, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633801, 633980, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 633981, 634220, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634221, 634740, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634741, 634860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 634861, 635160, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635161, 635680, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635681, 635800, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 635801, 636040, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636041, 636620, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636621, 636680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636681, 636980, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 636981, 637560, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637561, 637600, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637601, 637860, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 637861, 638480, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638481, 638500, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638501, 638800, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 638801, 639420, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639421, 639440, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639441, 639680, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 639681, 640300, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640301, 640360, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640361, 640620, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 640621, 641220, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641221, 641300, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641301, 641500, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 641501, 642100, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642101, 642240, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642241, 642440, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 642441, 643040, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643041, 643180, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643181, 643320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643321, 643920, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 643921, 644120, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644121, 644260, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644261, 644840, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 644841, 645060, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645061, 645140, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645141, 645720, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645721, 645980, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 645981, 646080, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646081, 646660, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646661, 646920, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646921, 646960, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 646961, 647540, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647541, 647860, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647861, 647900, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 647901, 648460, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648461, 648780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648781, 648800, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 648801, 649340, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649341, 649720, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649721, 649740, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 649741, 650280, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650281, 650600, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650601, 650680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 650681, 651160, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651161, 651540, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651541, 651620, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 651621, 652080, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652081, 652420, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652421, 652560, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652561, 652960, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 652961, 653360, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 653361, 653480, {20, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 653481, 653900, {20, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 653901, 654240, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654241, 654420, {0, 20, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654421, 654780, {0, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 654781, 655180, {1, 20, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655181, 655360, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655361, 655700, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 655701, 656060, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656061, 656300, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656301, 656580, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 656581, 657000, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657001, 657240, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657241, 657520, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657521, 657880, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 657881, 658180, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658181, 658400, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658401, 658820, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 658821, 659100, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659101, 659320, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659321, 659700, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 659701, 660040, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660041, 660200, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660201, 660640, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660641, 660980, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 660981, 661140, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661141, 661520, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661521, 661920, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 661921, 662020, {0, 1, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662021, 662460, {1, 1, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662461, 662860, {1, 0, 20} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662861, 662940, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 662941, 663340, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663341, 663800, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663801, 663820, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 663821, 664280, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664281, 664740, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664741, 664760, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 664761, 665160, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665161, 665640, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665641, 665680, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 665681, 666100, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666101, 666560, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666561, 666600, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666601, 666980, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 666981, 667440, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667441, 667540, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667541, 667920, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 667921, 668380, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668381, 668480, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668481, 668800, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 668801, 669260, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669261, 669420, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669421, 669740, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 669741, 670180, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670181, 670360, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670361, 670620, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 670621, 671060, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671061, 671300, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671301, 671560, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 671561, 672000, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672001, 672220, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672221, 672440, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672441, 672880, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 672881, 673160, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 673161, 673380, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 673381, 673800, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 673801, 674100, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674101, 674260, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674261, 674680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 674681, 675040, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675041, 675200, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675201, 675620, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675621, 675980, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 675981, 676080, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676081, 676500, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676501, 676920, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 676921, 677000, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677001, 677440, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677441, 677860, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677861, 677900, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 677901, 678300, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678301, 678800, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678801, 678820, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 678821, 679240, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 679241, 679720, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_332_NR5G_N077), 679721, 680000, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620000, 620380, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620381, 620600, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620601, 620680, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 620681, 621320, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621321, 621500, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621501, 621620, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 621621, 622180, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622181, 622420, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622421, 622560, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 622561, 623120, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623121, 623320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623321, 623500, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 623501, 624000, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624001, 624240, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624241, 624440, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624441, 624940, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 624941, 625140, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625141, 625380, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625381, 625800, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 625801, 626060, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626061, 626320, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626321, 626740, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626741, 626960, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 626961, 627240, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627241, 627620, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627621, 627880, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 627881, 628180, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628181, 628560, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628561, 628780, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 628781, 629120, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629121, 629420, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629421, 629700, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 629701, 630060, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630061, 630360, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630361, 630600, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 630601, 631000, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631001, 631240, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631241, 631520, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631521, 631940, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 631941, 632180, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632181, 632420, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632421, 632860, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 632861, 633060, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633061, 633340, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633341, 633800, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633801, 633980, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 633981, 634220, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634221, 634740, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634741, 634860, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 634861, 635160, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635161, 635680, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635681, 635800, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 635801, 636040, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636041, 636620, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636621, 636680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636681, 636980, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 636981, 637560, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637561, 637600, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637601, 637860, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 637861, 638480, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638481, 638500, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638501, 638800, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 638801, 639420, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639421, 639440, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639441, 639680, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 639681, 640300, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640301, 640360, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640361, 640620, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 640621, 641220, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641221, 641300, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641301, 641500, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 641501, 642100, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642101, 642240, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642241, 642440, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 642441, 643040, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643041, 643180, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643181, 643320, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643321, 643920, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 643921, 644120, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644121, 644260, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644261, 644840, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 644841, 645060, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645061, 645140, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645141, 645720, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645721, 645980, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 645981, 646080, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646081, 646660, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646661, 646920, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646921, 646960, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 646961, 647540, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647541, 647860, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647861, 647900, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 647901, 648460, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648461, 648780, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648781, 648800, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 648801, 649340, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649341, 649720, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649721, 649740, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 649741, 650280, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650281, 650600, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650601, 650680, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 650681, 651160, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651161, 651540, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651541, 651620, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 651621, 652080, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652081, 652420, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652421, 652560, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652561, 652960, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_333_NR5G_N078), 652961, 653333, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693334, 693394, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693395, 693714, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693715, 693794, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 693795, 694274, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694275, 694594, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694595, 694714, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 694715, 695214, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695215, 695534, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695535, 695654, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695655, 696094, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696095, 696414, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696415, 696594, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696595, 697034, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697035, 697354, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697355, 697534, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697535, 697914, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697915, 698214, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698215, 698474, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698475, 698834, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698835, 699154, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699155, 699414, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699415, 699734, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699735, 700034, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700035, 700334, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700335, 700654, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700655, 700974, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700975, 701294, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701295, 701554, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701555, 701834, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701835, 702214, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702215, 702474, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702475, 702774, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702775, 703154, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703155, 703374, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703375, 703654, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703655, 704094, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704095, 704294, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704295, 704594, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704595, 705034, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705035, 705194, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705195, 705454, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705455, 705974, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705975, 706114, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706115, 706394, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706395, 706914, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706915, 707014, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707015, 707274, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707275, 707834, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707835, 707934, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707935, 708214, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708215, 708774, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708775, 708834, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708835, 709074, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709075, 709714, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709715, 709754, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709755, 710014, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710015, 710654, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710655, 710894, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710895, 711574, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711575, 711594, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711595, 711834, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711835, 712474, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712475, 712534, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712535, 712694, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712695, 713394, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713395, 713454, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713455, 713634, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713635, 714294, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714295, 714414, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714415, 714514, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714515, 715214, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715215, 715334, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715335, 715454, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715455, 716114, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716115, 716274, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716275, 716334, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716335, 717034, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717035, 717214, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717215, 717254, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717255, 717934, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717935, 718134, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718135, 718154, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718155, 718854, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718855, 719074, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719075, 719094, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719095, 719734, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719735, 719954, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719955, 720034, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720035, 720674, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720675, 720874, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720875, 720954, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720955, 721554, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721555, 721754, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721755, 721894, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721895, 722494, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722495, 722694, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722695, 722834, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722835, 723374, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723375, 723574, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723575, 723774, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723775, 724314, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724315, 724494, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724495, 724714, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724715, 725194, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725195, 725374, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725375, 725654, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725655, 726134, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726135, 726314, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726315, 726574, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726575, 727014, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727015, 727194, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727195, 727534, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727535, 727954, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727955, 728114, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728115, 728454, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728455, 728834, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728835, 728994, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728995, 729394, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729395, 729774, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729775, 729934, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729935, 730334, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730335, 730654, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730655, 730814, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730815, 731274, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731275, 731594, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731595, 731734, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731735, 732214, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732215, 732474, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732475, 732614, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732615, 733154, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 733155, 733333, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695315, 695414, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695415, 695714, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 695715, 696234, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696235, 696354, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696355, 696654, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696655, 696894, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 696895, 697014, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697015, 697334, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697335, 697814, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697815, 697954, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 697955, 698274, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698275, 698474, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698475, 698614, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698615, 698974, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 698975, 699394, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699395, 699534, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699535, 699914, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 699915, 700054, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700055, 700194, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700195, 700594, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700595, 700974, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 700975, 701134, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701135, 701534, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701535, 701634, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701635, 701794, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 701795, 702234, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702235, 702574, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702575, 702714, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 702715, 703154, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703155, 703214, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703215, 703374, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703375, 703854, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 703855, 704154, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704155, 704314, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704315, 704794, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704795, 704974, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 704975, 705474, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705475, 705734, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705735, 705894, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 705895, 706374, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706375, 706414, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706415, 706554, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 706555, 707114, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707115, 707314, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707315, 707494, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707495, 707954, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 707955, 708034, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708035, 708154, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708155, 708734, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708735, 708894, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 708895, 709074, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709075, 709534, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709535, 709674, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709675, 709734, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 709735, 710354, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710355, 710474, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710475, 710674, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 710675, 711114, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711115, 711294, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711295, 711334, {1, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711335, 711994, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 711995, 712054, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712055, 712274, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712275, 712694, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712695, 712914, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 712915, 713614, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713615, 713634, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713635, 713854, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 713855, 714294, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714295, 714514, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714515, 714554, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 714555, 715214, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715215, 715254, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715255, 715454, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715455, 715874, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 715875, 716114, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716115, 716174, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716175, 716794, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716795, 716874, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 716875, 717034, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717035, 717454, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717455, 717694, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717695, 717814, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 717815, 718374, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718375, 718494, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718495, 718634, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 718635, 719034, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719035, 719294, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719295, 719434, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719435, 719954, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 719955, 720134, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720135, 720214, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720215, 720614, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720615, 720874, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 720875, 721054, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721055, 721554, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721555, 721754, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721755, 721814, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 721815, 722194, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722195, 722474, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722475, 722694, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 722695, 723134, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723135, 723374, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723375, 723394, {0, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723395, 723774, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 723775, 724054, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724055, 724314, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724315, 724714, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724715, 724994, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 724995, 725014, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725015, 725354, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725355, 725654, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725655, 725934, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 725935, 726294, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726295, 726574, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726575, 726634, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726635, 726934, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 726935, 727234, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727235, 727574, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727575, 727874, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 727875, 728174, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728175, 728274, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728275, 728514, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728515, 728834, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 728835, 729194, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729195, 729454, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729455, 729774, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729775, 729894, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 729895, 730094, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730095, 730414, {1, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730415, 730834, {1, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 730835, 731034, {1, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731035, 731354, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731355, 731514, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731515, 731694, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 731695, 732014, {20, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732015, 732454, {20, 1, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732455, 732614, {20, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732615, 732954, {0, 1, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 732955, 733154, {0, 0, 0} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 733155, 733274, {0, 0, 1} },
	{ CAM_RAT_BAND(CAM_RAT_7_NR5G, CAM_BAND_334_NR5G_N079), 733275, 733333, {1, 0, 1} },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_B_mode[] = {
	{
		sensor_imx754_setfile_B_mipi_channel,	ARRAY_SIZE(sensor_imx754_setfile_B_mipi_channel),
		sensor_imx754_setfile_B_mipi_setting,	ARRAY_SIZE(sensor_imx754_setfile_B_mipi_setting)
	},
};

/*************************************/
/* ========== DUMMY ================ */
/*************************************/
enum {
	CAM_TELE_SET_DUMMY = 0,
};

struct cam_sensor_i2c_reg_array MIPI_TELE_DUMMY_REG_ARRAY[] = {
};

static const struct cam_sensor_i2c_reg_setting sensor_tele_setfile_dummy[] = {
    { MIPI_TELE_DUMMY_REG_ARRAY, ARRAY_SIZE(MIPI_TELE_DUMMY_REG_ARRAY),
	  CAMERA_SENSOR_I2C_TYPE_WORD, CAMERA_SENSOR_I2C_TYPE_WORD, 0 }
};

static const struct cam_mipi_setting sensor_tele_setfile_dummy_mipi_setting[] = {
	{ "DUMMY Msps", 0,
	  sensor_tele_setfile_dummy, ARRAY_SIZE(sensor_tele_setfile_dummy) },
};

static const struct cam_mipi_cell_ratings sensor_tele_setfile_dummy_channel[] = {
	{ CAM_RAT_BAND(CAM_RAT_1_GSM, CAM_BAND_001_GSM_GSM850), 0, 0, {0} },
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_C_mode[] = {
	{
		sensor_tele_setfile_dummy_channel,	ARRAY_SIZE(sensor_tele_setfile_dummy_channel),
		sensor_tele_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_tele_setfile_dummy_mipi_setting)
	},
};

static const struct cam_mipi_sensor_mode sensor_tele_mipi_D_mode[] = {
	{
		sensor_tele_setfile_dummy_channel,	ARRAY_SIZE(sensor_tele_setfile_dummy_channel),
		sensor_tele_setfile_dummy_mipi_setting,	ARRAY_SIZE(sensor_tele_setfile_dummy_mipi_setting)
	},
};
#endif /* _CAM_SENSOR_ADAPTIVE_MIPI_IMX754_H_ */
